@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002-2020 Derick Rethans                               |
+   | Copyright (c) 2002-2021 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 1.01 of the Xdebug license,   |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -12,8 +12,6 @@
    | to obtain it through the world-wide-web, please send a note to       |
    | derick@xdebug.org so we can mail you a copy immediately.             |
    +----------------------------------------------------------------------+
-   | Authors: Derick Rethans <derick@xdebug.org>                          |
-   +----------------------------------------------------------------------+
  */
 
 #include "php_xdebug.h"
@@ -102,14 +100,14 @@ static void xdebug_func_format(char *buffer, size_t buffer_size, xdebug_func *fu
 
 	if (func->type == XFUNC_MEMBER) {
 		int func_len = strlen(func->function);
-		int len = ZSTR_LEN(func->class_name) + 2 + func_len;
+		int len = ZSTR_LEN(func->object_class) + 2 + func_len;
 
 		if (len + 1 > buffer_size) {
 			goto error;
 		}
-		memcpy(buffer, ZSTR_VAL(func->class_name), ZSTR_LEN(func->class_name));
-		memcpy(buffer + ZSTR_LEN(func->class_name), "->", 2);
-		memcpy(buffer + ZSTR_LEN(func->class_name) + 2, func->function, func_len);
+		memcpy(buffer, ZSTR_VAL(func->object_class), ZSTR_LEN(func->object_class));
+		memcpy(buffer + ZSTR_LEN(func->object_class), "->", 2);
+		memcpy(buffer + ZSTR_LEN(func->object_class) + 2, func->function, func_len);
 		buffer[len] = '\0';
 		return;
 	}
@@ -121,14 +119,22 @@ error:
 
 static void xdebug_build_fname_from_oparray(xdebug_func *tmp, zend_op_array *opa)
 {
-	int closure = 0;
+	int wrapped = 0;
 
 	memset(tmp, 0, sizeof(xdebug_func));
 
 	if (opa->function_name) {
 		if (opa->fn_flags & ZEND_ACC_CLOSURE) {
 			tmp->function = xdebug_wrap_closure_location_around_function_name(opa, STR_NAME_VAL(opa->function_name));
-			closure = 1;
+			wrapped = 1;
+#if PHP_VERSION_ID >= 70400
+		} else if (
+			opa->fn_flags & ZEND_ACC_TRAIT_CLONE
+			|| (opa->scope && opa->scope->ce_flags & ZEND_ACC_TRAIT)
+		) {
+			tmp->function = xdebug_wrap_location_around_function_name("trait-method", opa, STR_NAME_VAL(opa->function_name));
+			wrapped = 1;
+#endif
 		} else {
 			tmp->function = xdstrdup(STR_NAME_VAL(opa->function_name));
 		}
@@ -137,9 +143,9 @@ static void xdebug_build_fname_from_oparray(xdebug_func *tmp, zend_op_array *opa
 		tmp->type = XFUNC_MAIN;
 	}
 
-	if (opa->scope && !closure) {
+	if (opa->scope && !wrapped) {
 		tmp->type = XFUNC_MEMBER;
-		tmp->class_name = zend_string_copy(opa->scope->name);
+		tmp->object_class = zend_string_copy(opa->scope->name);
 	} else {
 		tmp->type = XFUNC_NORMAL;
 	}
@@ -154,8 +160,11 @@ static void xdebug_print_opcode_info(zend_execute_data *execute_data, const zend
 
 	xdebug_build_fname_from_oparray(&func_info, op_array);
 	xdebug_func_format(function_name, sizeof(function_name), &func_info);
-	if (func_info.class_name) {
-		zend_string_release(func_info.class_name);
+	if (func_info.object_class) {
+		zend_string_release(func_info.object_class);
+	}
+	if (func_info.scope_class) {
+		zend_string_release(func_info.scope_class);
 	}
 	if (func_info.function) {
 		xdfree(func_info.function);
@@ -556,8 +565,11 @@ static void prefill_from_oparray(zend_string *filename, zend_op_array *op_array)
 		xdebug_build_fname_from_oparray(&func_info, op_array);
 		xdebug_func_format(function_name, sizeof(function_name), &func_info);
 
-		if (func_info.class_name) {
-			zend_string_release(func_info.class_name);
+		if (func_info.object_class) {
+			zend_string_release(func_info.object_class);
+		}
+		if (func_info.scope_class) {
+			zend_string_release(func_info.scope_class);
 		}
 		if (func_info.function) {
 			xdfree(func_info.function);
@@ -567,6 +579,16 @@ static void prefill_from_oparray(zend_string *filename, zend_op_array *op_array)
 		xdebug_branch_find_paths(branch_info);
 		xdebug_branch_info_add_branches_and_paths(filename, (char*) function_name, branch_info);
 	}
+
+#if PHP_VERSION_ID >= 80100
+	if (!op_array->num_dynamic_func_defs) {
+		return;
+	}
+
+	for (i = 0; i < op_array->num_dynamic_func_defs; i++) {
+		prefill_from_oparray(filename, op_array->dynamic_func_defs[i]);
+	}
+#endif
 }
 
 static int prefill_from_function_table(zend_op_array *opa)
@@ -633,20 +655,20 @@ void xdebug_code_coverage_start_of_function(zend_op_array *op_array, char *funct
 	xdebug_path *path = xdebug_path_new(NULL);
 
 	xdebug_prefill_code_coverage(op_array);
-	xdebug_path_info_add_path_for_level(XG_COV(paths_stack), path, XG_BASE(level));
+	xdebug_path_info_add_path_for_level(XG_COV(paths_stack), path, XDEBUG_VECTOR_COUNT(XG_BASE(stack)));
 
-	if (XG_COV(branches).size == 0 || XG_BASE(level) >= XG_COV(branches).size) {
-		XG_COV(branches).size = XG_BASE(level) + 32;
+	if (XG_COV(branches).size == 0 || XDEBUG_VECTOR_COUNT(XG_BASE(stack)) >= XG_COV(branches).size) {
+		XG_COV(branches).size = XDEBUG_VECTOR_COUNT(XG_BASE(stack)) + 32;
 		XG_COV(branches).last_branch_nr = realloc(XG_COV(branches).last_branch_nr, sizeof(int) * XG_COV(branches.size));
 	}
 
-	XG_COV(branches).last_branch_nr[XG_BASE(level)] = -1;
+	XG_COV(branches).last_branch_nr[XDEBUG_VECTOR_COUNT(XG_BASE(stack))] = -1;
 }
 
 void xdebug_code_coverage_end_of_function(zend_op_array *op_array, zend_string *filename, char *function_name)
 {
 	xdebug_str str = XDEBUG_STR_INITIALIZER;
-	xdebug_path *path = xdebug_path_info_get_path_for_level(XG_COV(paths_stack), XG_BASE(level));
+	xdebug_path *path = xdebug_path_info_get_path_for_level(XG_COV(paths_stack), XDEBUG_VECTOR_COUNT(XG_BASE(stack)));
 
 	if (!path || !path->elements) {
 		return;
@@ -686,13 +708,13 @@ PHP_FUNCTION(xdebug_start_code_coverage)
 
 PHP_FUNCTION(xdebug_stop_code_coverage)
 {
-	zend_long cleanup = 1;
+	zend_bool cleanup = 1;
 
 	if (!XDEBUG_MODE_IS(XDEBUG_MODE_COVERAGE)) {
 		RETURN_FALSE;
 	}
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "|l", &cleanup) == FAILURE) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "|b", &cleanup) == FAILURE) {
 		return;
 	}
 	if (!XG_COV(code_coverage_active)) {
@@ -700,10 +722,14 @@ PHP_FUNCTION(xdebug_stop_code_coverage)
 	}
 
 	if (cleanup) {
-		zend_string_release(XG_COV(previous_filename));
+		if (XG_COV(previous_filename)) {
+			zend_string_release(XG_COV(previous_filename));
+		}
 		XG_COV(previous_filename) = NULL;
 		XG_COV(previous_file) = NULL;
-		zend_string_release(XG_COV(previous_mark_filename));
+		if (XG_COV(previous_mark_filename)) {
+			zend_string_release(XG_COV(previous_mark_filename));
+		}
 		XG_COV(previous_mark_filename) = NULL;
 		XG_COV(previous_mark_file) = NULL;
 		xdebug_hash_destroy(XG_COV(code_coverage_info));
@@ -971,8 +997,11 @@ int xdebug_coverage_execute_ex(function_stack_entry *fse, zend_op_array *op_arra
 		*tmp_function_name = xdstrdup(buffer);
 		xdebug_code_coverage_start_of_function(op_array, *tmp_function_name);
 
-		if (func_info.class_name) {
-			zend_string_release(func_info.class_name);
+		if (func_info.object_class) {
+			zend_string_release(func_info.object_class);
+		}
+		if (func_info.scope_class) {
+			zend_string_release(func_info.scope_class);
 		}
 		if (func_info.function) {
 			xdfree(func_info.function);
@@ -1094,6 +1123,7 @@ void xdebug_coverage_minit(INIT_FUNC_ARGS)
 	xdebug_set_opcode_handler(ZEND_FETCH_CONSTANT, xdebug_common_override_handler);
 	xdebug_set_opcode_handler(ZEND_FETCH_CLASS_CONSTANT, xdebug_common_override_handler);
 	xdebug_set_opcode_handler(ZEND_CONCAT, xdebug_common_override_handler);
+	xdebug_set_opcode_handler(ZEND_FAST_CONCAT, xdebug_common_override_handler);
 	xdebug_set_opcode_handler(ZEND_ISSET_ISEMPTY_DIM_OBJ, xdebug_common_override_handler);
 	xdebug_set_opcode_handler(ZEND_ISSET_ISEMPTY_PROP_OBJ, xdebug_common_override_handler);
 	xdebug_set_opcode_handler(ZEND_CASE, xdebug_common_override_handler);
