@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002-2020 Derick Rethans                               |
+   | Copyright (c) 2002-2021 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 1.01 of the Xdebug license,   |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -12,8 +12,6 @@
    | to obtain it through the world-wide-web, please send a note to       |
    | derick@xdebug.org so we can mail you a copy immediately.             |
    +----------------------------------------------------------------------+
-   | Authors: Derick Rethans <derick@xdebug.org>                          |
-   +----------------------------------------------------------------------+
  */
 
 #include "php_xdebug.h"
@@ -41,9 +39,11 @@ void xdebug_init_debugger_globals(xdebug_debugger_globals_t *xg)
 	xg->context.list.last_filename = NULL;
 	xg->context.list.last_line = 0;
 	xg->context.do_break     = 0;
+	xg->context.pending_breakpoint = NULL;
 	xg->context.do_step      = 0;
 	xg->context.do_next      = 0;
 	xg->context.do_finish    = 0;
+	xg->context.do_connect_to_client = 0;
 
 	xg->remote_connection_enabled = 0;
 	xg->remote_connection_pid     = 0;
@@ -54,6 +54,11 @@ void xdebug_init_debugger_globals(xdebug_debugger_globals_t *xg)
 		xdebug_orig_ub_write = sapi_module.ub_write;
 		sapi_module.ub_write = xdebug_ub_write;
 	}
+
+	/* Statistics and diagnostics */
+	xg->context.connected_hostname = NULL;
+	xg->context.connected_port = 0;
+	xg->context.detached_message = NULL;
 }
 
 static char *xdebug_debugger_get_ide_key(void)
@@ -91,6 +96,25 @@ int xdebug_debugger_bailout_if_no_exec_requested(void)
 	return 0;
 }
 
+static void register_compiled_variables(void)
+{
+	function_stack_entry *loop_fse = XDEBUG_VECTOR_TAIL(XG_BASE(stack));
+	int                   i;
+
+	for (i = 0; i < XDEBUG_VECTOR_COUNT(XG_BASE(stack)); i++, loop_fse--) {
+		if (loop_fse->declared_vars) {
+			continue;
+		}
+
+		if (loop_fse->user_defined == XDEBUG_BUILT_IN) {
+			continue;
+		}
+
+		xdebug_lib_register_compiled_variables(loop_fse, loop_fse->op_array);
+	}
+}
+
+
 void xdebug_debugger_set_program_name(zend_string *filename)
 {
 	if (!XG_DBG(context).program_name) {
@@ -142,7 +166,7 @@ int xdebug_do_eval(char *eval_string, zval *ret_zval)
 #if PHP_VERSION_ID < 80000
 	int                old_track_errors;
 #endif
-	int                res = FAILURE;
+	int                res = 1;
 	zend_execute_data *original_execute_data = EG(current_execute_data);
 	int                original_no_extensions = EG(no_extensions);
 	zend_object       *original_exception = EG(exception);
@@ -167,12 +191,17 @@ int xdebug_do_eval(char *eval_string, zval *ret_zval)
 
 	/* Do evaluation */
 	zend_first_try {
-		res = zend_eval_string(eval_string, ret_zval, (char*) "xdebug://debug-eval");
+		res = (zend_eval_string(eval_string, ret_zval, (char*) "xdebug://debug-eval") == SUCCESS);
 	} zend_end_try();
 
 	/* FIXME: Bubble up exception message to DBGp return packet */
 	if (EG(exception)) {
-		res = FAILURE;
+#if PHP_VERSION_ID >= 80000
+		if (!res) {
+			zend_clear_exception();
+		}
+#endif
+		res = 0;
 	}
 
 	/* Clean up */
@@ -200,11 +229,23 @@ void xdebug_debugger_statement_call(zend_string *filename, int lineno)
 	int                   level = 0;
 	int                   func_nr = 0;
 
+	if (XG_DBG(context).do_connect_to_client) {
+		XG_DBG(context).do_connect_to_client = 0;
+
+		if (!xdebug_is_debug_connection_active()) {
+			xdebug_debug_init_if_requested_on_xdebug_break();
+			register_compiled_variables();
+		}
+	}
+
 	if (xdebug_is_debug_connection_active()) {
 		if (XG_DBG(context).do_break) {
-			XG_DBG(context).do_break = 0;
+			xdebug_brk_info *brk_info = XG_DBG(context).pending_breakpoint;
 
-			if (!XG_DBG(context).handler->remote_breakpoint(&(XG_DBG(context)), XG_BASE(stack), filename, lineno, XDEBUG_BREAK, NULL, 0, NULL)) {
+			XG_DBG(context).do_break = 0;
+			XG_DBG(context).pending_breakpoint = NULL;
+
+			if (!XG_DBG(context).handler->remote_breakpoint(&(XG_DBG(context)), XG_BASE(stack), filename, lineno, XDEBUG_BREAK, NULL, 0, NULL, brk_info)) {
 				xdebug_mark_debug_connection_not_active();
 				return;
 			}
@@ -230,7 +271,7 @@ void xdebug_debugger_statement_call(zend_string *filename, int lineno)
 		) {
 			XG_DBG(context).do_finish = 0;
 
-			if (!XG_DBG(context).handler->remote_breakpoint(&(XG_DBG(context)), XG_BASE(stack), filename, lineno, XDEBUG_STEP, NULL, 0, NULL)) {
+			if (!XG_DBG(context).handler->remote_breakpoint(&(XG_DBG(context)), XG_BASE(stack), filename, lineno, XDEBUG_STEP, NULL, 0, NULL, NULL)) {
 				xdebug_mark_debug_connection_not_active();
 				return;
 			}
@@ -241,7 +282,7 @@ void xdebug_debugger_statement_call(zend_string *filename, int lineno)
 		if (XG_DBG(context).do_next && XG_DBG(context).next_level >= level) {
 			XG_DBG(context).do_next = 0;
 
-			if (!XG_DBG(context).handler->remote_breakpoint(&(XG_DBG(context)), XG_BASE(stack), filename, lineno, XDEBUG_STEP, NULL, 0, NULL)) {
+			if (!XG_DBG(context).handler->remote_breakpoint(&(XG_DBG(context)), XG_BASE(stack), filename, lineno, XDEBUG_STEP, NULL, 0, NULL, NULL)) {
 				xdebug_mark_debug_connection_not_active();
 				return;
 			}
@@ -252,7 +293,7 @@ void xdebug_debugger_statement_call(zend_string *filename, int lineno)
 		if (XG_DBG(context).do_step) {
 			XG_DBG(context).do_step = 0;
 
-			if (!XG_DBG(context).handler->remote_breakpoint(&(XG_DBG(context)), XG_BASE(stack), filename, lineno, XDEBUG_STEP, NULL, 0, NULL)) {
+			if (!XG_DBG(context).handler->remote_breakpoint(&(XG_DBG(context)), XG_BASE(stack), filename, lineno, XDEBUG_STEP, NULL, 0, NULL, NULL)) {
 				xdebug_mark_debug_connection_not_active();
 				return;
 			}
@@ -278,13 +319,13 @@ void xdebug_debugger_statement_call(zend_string *filename, int lineno)
 
 						/* Remember error reporting level */
 						res = xdebug_do_eval(extra_brk_info->condition, &retval);
-						if (res == SUCCESS) {
+						if (res) {
 							break_ok = Z_TYPE(retval) == IS_TRUE;
 							zval_dtor(&retval);
 						}
 					}
 					if (break_ok && xdebug_handle_hit_value(extra_brk_info)) {
-						if (!XG_DBG(context).handler->remote_breakpoint(&(XG_DBG(context)), XG_BASE(stack), filename, lineno, XDEBUG_BREAK, NULL, 0, NULL)) {
+						if (!XG_DBG(context).handler->remote_breakpoint(&(XG_DBG(context)), XG_BASE(stack), filename, lineno, XDEBUG_BREAK, NULL, 0, NULL, extra_brk_info)) {
 							xdebug_mark_debug_connection_not_active();
 							break;
 						}
@@ -334,12 +375,15 @@ void xdebug_debugger_throw_exception_hook(zval *exception, zval *file, zval *lin
 		}
 #endif
 		if (exception_breakpoint_found && xdebug_handle_hit_value(extra_brk_info)) {
-			if (!XG_DBG(context).handler->remote_breakpoint(
-				&(XG_DBG(context)), XG_BASE(stack),
-				Z_STR_P(file), Z_LVAL_P(line), XDEBUG_BREAK,
-				(char*) STR_NAME_VAL(exception_ce->name),
-				code_str ? code_str : ((code && Z_TYPE_P(code) == IS_STRING) ? Z_STRVAL_P(code) : NULL),
-				message ? Z_STRVAL_P(message) : "")
+			if (
+				!XG_DBG(context).handler->remote_breakpoint(
+					&(XG_DBG(context)), XG_BASE(stack),
+					Z_STR_P(file), Z_LVAL_P(line), XDEBUG_BREAK,
+					(char*) STR_NAME_VAL(exception_ce->name),
+					code_str ? code_str : ((code && Z_TYPE_P(code) == IS_STRING) ? Z_STRVAL_P(code) : NULL),
+					message ? Z_STRVAL_P(message) : "",
+					extra_brk_info
+				)
 			) {
 				xdebug_mark_debug_connection_not_active();
 			}
@@ -370,7 +414,7 @@ void xdebug_debugger_error_cb(zend_string *error_filename, int error_lineno, int
 			if (xdebug_handle_hit_value(extra_brk_info)) {
 				char *type_str = xdebug_sprintf("%ld", type);
 
-				if (!XG_DBG(context).handler->remote_breakpoint(&(XG_DBG(context)), XG_BASE(stack), error_filename, error_lineno, XDEBUG_BREAK, error_type_str, type_str, buffer)) {
+				if (!XG_DBG(context).handler->remote_breakpoint(&(XG_DBG(context)), XG_BASE(stack), error_filename, error_lineno, XDEBUG_BREAK, error_type_str, type_str, buffer, extra_brk_info)) {
 					xdebug_mark_debug_connection_not_active();
 				}
 
@@ -394,11 +438,12 @@ static int handle_breakpoints(function_stack_entry *fse, int breakpoint_type)
 			if (!extra_brk_info->disabled && (extra_brk_info->function_break_type == breakpoint_type)) {
 				if (xdebug_handle_hit_value(extra_brk_info)) {
 					if (fse->user_defined == XDEBUG_BUILT_IN || (breakpoint_type == XDEBUG_BREAKPOINT_TYPE_RETURN)) {
-						if (!XG_DBG(context).handler->remote_breakpoint(&(XG_DBG(context)), XG_BASE(stack), fse->filename, fse->lineno, XDEBUG_BREAK, NULL, 0, NULL)) {
+						if (!XG_DBG(context).handler->remote_breakpoint(&(XG_DBG(context)), XG_BASE(stack), fse->filename, fse->lineno, XDEBUG_BREAK, NULL, 0, NULL, extra_brk_info)) {
 							return 0;
 						}
 					} else {
 						XG_DBG(context).do_break = 1;
+						XG_DBG(context).pending_breakpoint = extra_brk_info;
 					}
 				}
 			}
@@ -406,11 +451,13 @@ static int handle_breakpoints(function_stack_entry *fse, int breakpoint_type)
 	}
 	/* class->function breakpoints */
 	else if (fse->function.type == XFUNC_MEMBER || fse->function.type == XFUNC_STATIC_MEMBER) {
-		/* We intentionally do not use xdebug_sprintf because it can create a bottleneck in large
-		   codebases due to setlocale calls. We don't care about the locale here. */
-		tmp_len = ZSTR_LEN(fse->function.class_name) + strlen(fse->function.function) + 3;
+		/* Using strlen(ZSTR_VAL(...)) here to cut of the string at the first \0, which is needed
+		 * for anonymous classes, in combination with the snprintf() below */
+		tmp_len = strlen(ZSTR_VAL(fse->function.object_class)) + strlen(fse->function.function) + 3;
 		tmp_name = xdmalloc(tmp_len);
-		snprintf(tmp_name, tmp_len, "%s::%s", ZSTR_VAL(fse->function.class_name), fse->function.function);
+		/* We intentionally do not use xdebug_sprintf because it can create a bottleneck in large
+		 * codebases due to setlocale calls. We don't care about the locale here. */
+		snprintf(tmp_name, tmp_len, "%s::%s", ZSTR_VAL(fse->function.object_class), fse->function.function);
 
 		if (xdebug_hash_find(XG_DBG(context).function_breakpoints, tmp_name, tmp_len - 1, (void *) &extra_brk_info)) {
 			/* Yup, breakpoint found, call handler if the breakpoint is not
@@ -418,11 +465,12 @@ static int handle_breakpoints(function_stack_entry *fse, int breakpoint_type)
 			if (!extra_brk_info->disabled && (extra_brk_info->function_break_type == breakpoint_type)) {
 				if (xdebug_handle_hit_value(extra_brk_info)) {
 					if (fse->user_defined == XDEBUG_BUILT_IN || (breakpoint_type == XDEBUG_BREAKPOINT_TYPE_RETURN)) {
-						if (!XG_DBG(context).handler->remote_breakpoint(&(XG_DBG(context)), XG_BASE(stack), fse->filename, fse->lineno, XDEBUG_BREAK, NULL, 0, NULL)) {
+						if (!XG_DBG(context).handler->remote_breakpoint(&(XG_DBG(context)), XG_BASE(stack), fse->filename, fse->lineno, XDEBUG_BREAK, NULL, 0, NULL, extra_brk_info)) {
 							return 0;
 						}
 					} else {
 						XG_DBG(context).do_break = 1;
+						XG_DBG(context).pending_breakpoint = extra_brk_info;
 					}
 				}
 			}
@@ -525,7 +573,7 @@ void xdebug_debugger_rinit(void)
 			)
 			&& !SG(headers_sent)
 		) {
-			xdebug_setcookie("XDEBUG_SESSION", sizeof("XDEBUG_SESSION"), (char*) "", 0, time(NULL) + XDEBUG_COOKIE_EXPIRE_TIME, "/", 1, NULL, 0, 0, 1, 0);
+			xdebug_setcookie("XDEBUG_SESSION", sizeof("XDEBUG_SESSION"), (char*) "", 0, 0, "/", 1, NULL, 0, 0, 1, 0);
 			XG_DBG(no_exec) = 1;
 		}
 		zend_string_release(stop_no_exec);
@@ -544,9 +592,16 @@ void xdebug_debugger_rinit(void)
 	XG_DBG(context).list.last_filename = NULL;
 	XG_DBG(context).list.last_line = 0;
 	XG_DBG(context).do_break       = 0;
+	XG_DBG(context).pending_breakpoint = NULL;
 	XG_DBG(context).do_step        = 0;
 	XG_DBG(context).do_next        = 0;
 	XG_DBG(context).do_finish      = 0;
+	XG_DBG(context).do_connect_to_client = 0;
+
+	/* Statistics and diagnostics */
+	XG_DBG(context).connected_hostname = NULL;
+	XG_DBG(context).connected_port = 0;
+	XG_DBG(context).detached_message = NULL;
 }
 
 void xdebug_debugger_post_deactivate(void)
@@ -571,6 +626,16 @@ void xdebug_debugger_post_deactivate(void)
 
 	xdebug_hash_destroy(XG_DBG(breakable_lines_map));
 	XG_DBG(breakable_lines_map) = NULL;
+
+	if (XG_DBG(context).connected_hostname) {
+		xdfree(XG_DBG(context).connected_hostname);
+		XG_DBG(context).connected_hostname = NULL;
+	}
+
+	if (XG_DBG(context).detached_message) {
+		xdfree(XG_DBG(context).detached_message);
+		XG_DBG(context).detached_message = NULL;
+	}
 }
 
 xdebug_set *xdebug_debugger_get_breakable_lines_from_oparray(zend_op_array *opa)
@@ -641,6 +706,16 @@ static void add_function_to_lines_list(xdebug_lines_list *lines_list, zend_op_ar
 	}
 	lines_list->functions[lines_list->count] = map_item;
 	lines_list->count++;
+
+#if PHP_VERSION_ID >= 80100
+	if (opa->num_dynamic_func_defs) {
+		uint32_t i;
+
+		for (i = 0; i < opa->num_dynamic_func_defs; i++) {
+			add_function_to_lines_list(lines_list, opa->dynamic_func_defs[i]);
+		}
+	}
+#endif
 }
 /* }}} */
 
@@ -771,7 +846,6 @@ void xdebug_debugger_restart_if_pid_changed()
 	}
 }
 
-
 PHP_FUNCTION(xdebug_break)
 {
 	RETURN_FALSE_IF_MODE_IS_NOT(XDEBUG_MODE_STEP_DEBUG);
@@ -782,7 +856,20 @@ PHP_FUNCTION(xdebug_break)
 		RETURN_FALSE;
 	}
 
+	register_compiled_variables();
+
 	XG_DBG(context).do_break = 1;
+	XG_DBG(context).pending_breakpoint = NULL;
+
+	RETURN_TRUE;
+}
+
+PHP_FUNCTION(xdebug_connect_to_client)
+{
+	RETURN_FALSE_IF_MODE_IS_NOT(XDEBUG_MODE_STEP_DEBUG);
+
+	XG_DBG(context).do_connect_to_client = 1;
+
 	RETURN_TRUE;
 }
 
@@ -790,3 +877,29 @@ PHP_FUNCTION(xdebug_is_debugger_active)
 {
 	RETURN_BOOL(xdebug_is_debug_connection_active());
 }
+
+PHP_FUNCTION(xdebug_notify)
+{
+	function_stack_entry *fse;
+	zval *data;
+
+	RETURN_FALSE_IF_MODE_IS_NOT(XDEBUG_MODE_STEP_DEBUG);
+
+	if (!xdebug_is_debug_connection_active()) {
+		RETURN_FALSE;
+	}
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "z", &data) == FAILURE) {
+		return;
+	}
+
+	fse = xdebug_get_stack_frame(0);
+
+	XG_DBG(context).handler->user_notification(
+		&(XG_DBG(context)),
+		fse->filename, fse->lineno,
+		data
+	);
+
+	RETURN_TRUE;
+}
