@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002-2020 Derick Rethans                               |
+   | Copyright (c) 2002-2021 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 1.01 of the Xdebug license,   |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -12,8 +12,6 @@
    | to obtain it through the world-wide-web, please send a note to       |
    | derick@xdebug.org so we can mail you a copy immediately.             |
    +----------------------------------------------------------------------+
-   | Authors: Derick Rethans <derick@xdebug.org>                          |
-   +----------------------------------------------------------------------+
  */
 
 #ifndef __HAVE_XDEBUG_HANDLERS_H__
@@ -57,8 +55,13 @@ struct _xdebug_brk_admin {
 	char *key;
 };
 
+#define XDEBUG_NORMAL                   0x01
+#define XDEBUG_CLOUD                    0x02
+#define XDEBUG_CLOUD_FROM_TRIGGER_VALUE 0x03
+
 struct _xdebug_con {
 	int                    socket;
+	int                    host_type; /* XDEBUG_NORMAL, XDEBUG_CLOUD, XDEBUG_CLOUD_FROM_TRIGGER_VALUE */
 	void                  *options;
 	xdebug_remote_handler *handler;
 	fd_buf                *buffer;
@@ -71,11 +74,13 @@ struct _xdebug_con {
 	xdebug_hash           *exception_breakpoints;
 	xdebug_debug_list      list;
 	int                    do_break;
+	xdebug_brk_info       *pending_breakpoint;
 
 	int                    do_step;
 	int                    do_next;
 	int                    next_level;
 	int                    do_finish;
+	int                    do_connect_to_client;
 	int                    finish_level;
 	int                    finish_func_nr;
 
@@ -83,6 +88,12 @@ struct _xdebug_con {
 	int                    inhibit_notifications;
 
 	int                    resolved_breakpoints;
+	int                    breakpoint_details;
+
+	/* Statistics and diagnostics */
+	char *connected_hostname;
+	int   connected_port;
+	char *detached_message;
 };
 
 #define XDEBUG_BRK_UNRESOLVED     0
@@ -130,7 +141,7 @@ struct _xdebug_remote_handler {
 
 	/* Breakpoints */
 	int (*break_on_line)(xdebug_con *h, xdebug_brk_info *brk, zend_string *filename, int lineno);
-	int (*remote_breakpoint)(xdebug_con *h, xdebug_vector *stack, zend_string *filename, long lineno, int type, char *exception, char *code, const char *message);
+	int (*remote_breakpoint)(xdebug_con *h, xdebug_vector *stack, zend_string *filename, long lineno, int type, char *exception, char *code, const char *message, xdebug_brk_info *brk_info);
 	int (*resolve_breakpoints)(xdebug_con *h, zend_string *opa);
 
 	/* Output redirection */
@@ -138,6 +149,7 @@ struct _xdebug_remote_handler {
 
 	/* Notifications & Logging */
 	int (*remote_notification)(xdebug_con *h, zend_string *file, long lineno, int type, char *type_string, char *message);
+	int (*user_notification)(xdebug_con *h, zend_string *filename, long lineno, zval *data);
 
 	/* Eval ID registration and removal */
 	int (*register_eval_id)(xdebug_con *h, function_stack_entry *fse);
