@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002-2020 Derick Rethans                               |
+   | Copyright (c) 2002-2021 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 1.01 of the Xdebug license,   |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -12,13 +12,14 @@
    | to obtain it through the world-wide-web, please send a note to       |
    | derick@xdebug.org so we can mail you a copy immediately.             |
    +----------------------------------------------------------------------+
-   | Authors: Derick Rethans <derick@xdebug.org>                          |
-   +----------------------------------------------------------------------+
  */
 
 #ifndef __HAVE_USEFULSTUFF_H__
 #define __HAVE_USEFULSTUFF_H__
 
+#include "php_xdebug.h"
+#include "src/lib/compat.h"
+
 typedef struct xdebug_arg {
 	int    c;
 	char **args;
@@ -31,6 +32,7 @@ xdebug_str* xdebug_join(const char *delim, xdebug_arg *args, int begin, int end)
 void xdebug_explode(const char *delim, const char *str, xdebug_arg *args, int limit);
 const char* xdebug_memnstr(const char *haystack, const char *needle, int needle_len, const char *end);
 char* xdebug_strrstr(const char* haystack, const char* needle);
+char *xdebug_trim(const char *str);
 char *xdebug_path_to_url(zend_string *fileurl);
 char *xdebug_path_from_url(zend_string *fileurl);
 FILE *xdebug_fopen(char *fname, const char *mode, const char *extension, char **new_fname);
