@@ -31,27 +31,28 @@ static int xdebug_array_element_export(zval *zv_nptr, zend_ulong index_key, zend
 		options->runtime[level].current_element_nr < options->runtime[level].end_element_nr)
 	{
 		if (HASH_KEY_IS_NUMERIC(hash_key)) { /* numeric key */
-			xdebug_str_add_fmt(str, XDEBUG_INT_FMT " => ", index_key);
+			xdebug_str_add(str, xdebug_sprintf(XDEBUG_INT_FMT " => ", index_key), 1);
 		} else { /* string key */
-			zend_string *tmp, *tmp2;
+			size_t newlen = 0;
+			char *tmp, *tmp2;
 
-			tmp = php_str_to_str(ZSTR_VAL(hash_key), ZSTR_LEN(hash_key), (char*) "'", 1, (char*) "\\'", 2);
-			tmp2 = php_str_to_str(ZSTR_VAL(tmp), ZSTR_LEN(tmp), (char*) "\0", 1, (char*) "\\0", 2);
+			tmp = xdebug_str_to_str((char*) HASH_APPLY_KEY_VAL(hash_key), HASH_APPLY_KEY_LEN(hash_key), "'", 1, "\\'", 2, &newlen);
+			tmp2 = xdebug_str_to_str(tmp, newlen - 1, "\0", 1, "\\0", 2, &newlen);
 			if (tmp) {
-				zend_string_release(tmp);
+				efree(tmp);
 			}
-			xdebug_str_addc(str, '\'');
+			xdebug_str_addl(str, "'", 1, 0);
 			if (tmp2) {
-				xdebug_str_add_zstr(str, tmp2);
-				zend_string_release(tmp2);
+				xdebug_str_addl(str, tmp2, newlen, 0);
+				efree(tmp2);
 			}
-			xdebug_str_add_literal(str, "' => ");
+			xdebug_str_add(str, "' => ", 0);
 		}
 		xdebug_var_export_line(zv, str, level + 2, debug_zval, options);
-		xdebug_str_add_literal(str, ", ");
+		xdebug_str_addl(str, ", ", 2, 0);
 	}
 	if (options->runtime[level].current_element_nr == options->runtime[level].end_element_nr) {
-		xdebug_str_add_literal(str, "..., ");
+		xdebug_str_addl(str, "..., ", 5, 0);
 	}
 	options->runtime[level].current_element_nr++;
 	return 0;
@@ -79,16 +80,16 @@ static int xdebug_object_element_export(zval *object, zval *zv_nptr, zend_ulong
 				xdebug_str_addc(str, ' ');
 				xdebug_str_add_str(str, property_type);
 			}
-			xdebug_str_add_literal(str, " $");
+			xdebug_str_addl(str, " $", 2, 0);
 			if (strcmp(modifier, "private") != 0 || strcmp(class_name, prop_class_name) == 0) {
 				xdebug_str_add_str(str, property_name);
-				xdebug_str_add_literal(str, " = ");
+				xdebug_str_addl(str, " = ", 3, 0);
 			} else {
 				xdebug_str_addc(str, '{');
 				xdebug_str_add(str, prop_class_name, 0);
 				xdebug_str_addc(str, '}');
 				xdebug_str_add_str(str, property_name);
-				xdebug_str_add_literal(str, " = ");
+				xdebug_str_addl(str, " = ", 3, 0);
 			}
 
 			if (property_type) {
@@ -97,13 +98,13 @@ static int xdebug_object_element_export(zval *object, zval *zv_nptr, zend_ulong
 			xdebug_str_free(property_name);
 			xdfree(prop_class_name);
 		} else {
-			xdebug_str_add_fmt(str, "public $%d = ", index_key);
+			xdebug_str_add(str, xdebug_sprintf("public $%d = ", index_key), 1);
 		}
 		xdebug_var_export_line(zv, str, level + 2, debug_zval, options);
-		xdebug_str_add_literal(str, "; ");
+		xdebug_str_addl(str, "; ", 2, 0);
 	}
 	if (options->runtime[level].current_element_nr == options->runtime[level].end_element_nr) {
-		xdebug_str_add_literal(str, "...; ");
+		xdebug_str_addl(str, "...; ", 5, 0);
 	}
 	options->runtime[level].current_element_nr++;
 	return 0;
@@ -112,6 +113,7 @@ static int xdebug_object_element_export(zval *object, zval *zv_nptr, zend_ulong
 void xdebug_var_export_line(zval **struc, xdebug_str *str, int level, int debug_zval, xdebug_var_export_options *options)
 {
 	HashTable *myht;
+	char*     tmp_str;
 #if PHP_VERSION_ID < 70400
 	int       is_temp;
 #endif
@@ -119,81 +121,72 @@ void xdebug_var_export_line(zval **struc, xdebug_str *str, int level, int debug_
 	zend_string *key;
 	zval *val;
 	zval *tmpz;
-	int   z_type;
 
 	if (!struc || !(*struc)) {
 		return;
 	}
 
-	z_type = Z_TYPE_P(*struc);
-
 	if (debug_zval) {
 		xdebug_add_variable_attributes(str, *struc, XDEBUG_VAR_ATTR_TEXT);
 	}
-	if (z_type == IS_INDIRECT) {
+	if (Z_TYPE_P(*struc) == IS_INDIRECT) {
 		tmpz = Z_INDIRECT_P(*struc);
 		struc = &tmpz;
-		z_type = Z_TYPE_P(*struc);
 	}
-	if (z_type == IS_REFERENCE) {
+	if (Z_TYPE_P(*struc) == IS_REFERENCE) {
 		tmpz = &((*struc)->value.ref->val);
 		struc = &tmpz;
-		z_type = Z_TYPE_P(*struc);
 	}
 
-	switch (z_type) {
+	switch (Z_TYPE_P(*struc)) {
 		case IS_TRUE:
-			xdebug_str_add_literal(str, "TRUE");
-			break;
-
 		case IS_FALSE:
-			xdebug_str_add_literal(str, "FALSE");
+			xdebug_str_add(str, xdebug_sprintf("%s", Z_TYPE_P(*struc) == IS_TRUE ? "TRUE" : "FALSE"), 1);
 			break;
 
 		case IS_NULL:
-			xdebug_str_add_literal(str, "NULL");
+			xdebug_str_addl(str, "NULL", 4, 0);
 			break;
 
 		case IS_LONG:
-			xdebug_str_add_fmt(str, XDEBUG_INT_FMT, Z_LVAL_P(*struc));
+			xdebug_str_add(str, xdebug_sprintf(XDEBUG_INT_FMT, Z_LVAL_P(*struc)), 1);
 			break;
 
 		case IS_DOUBLE:
-			xdebug_str_add_fmt(str, "%.*G", (int) EG(precision), Z_DVAL_P(*struc));
+			xdebug_str_add(str, xdebug_sprintf("%.*G", (int) EG(precision), Z_DVAL_P(*struc)), 1);
 			break;
 
 		case IS_STRING: {
+			zend_string *i_string = zend_string_init(Z_STRVAL_P(*struc), Z_STRLEN_P(*struc), 0);
 			zend_string *tmp_zstr;
 
 #if PHP_VERSION_ID >= 70300
-			tmp_zstr = php_addcslashes(Z_STR_P(*struc), (char*) "'\\\0..\37", 7);
+			tmp_zstr = php_addcslashes(i_string, (char*) "'\\\0..\37", 7);
 #else
-			tmp_zstr = php_addcslashes(Z_STR_P(*struc), 0, (char*) "'\\\0..\37", 7);
+			tmp_zstr = php_addcslashes(i_string, 0, (char*) "'\\\0..\37", 7);
 #endif
 
-			if (options->no_decoration) {
-				xdebug_str_add_zstr(str, tmp_zstr);
-			} else if ((size_t) Z_STRLEN_P(*struc) <= (size_t) options->max_data) {
-				xdebug_str_addc(str, '\'');
-				xdebug_str_add_zstr(str, tmp_zstr);
-				xdebug_str_addc(str, '\'');
-			} else {
-				xdebug_str_addc(str, '\'');
-				xdebug_str_addl(str, ZSTR_VAL(tmp_zstr), options->max_data, 0);
-				xdebug_str_addc(str, '\'');
-			}
+			tmp_str = estrndup(tmp_zstr->val, tmp_zstr->len);
 			zend_string_release(tmp_zstr);
+			zend_string_release(i_string);
+
+			if (options->no_decoration) {
+				xdebug_str_add(str, tmp_str, 0);
+			} else if ((size_t) Z_STRLEN_P(*struc) <= (size_t) options->max_data) {
+				xdebug_str_add(str, xdebug_sprintf("'%s'", tmp_str), 1);
+			} else {
+				xdebug_str_addl(str, "'", 1, 0);
+				xdebug_str_addl(str, xdebug_sprintf("%s", tmp_str), options->max_data, 1);
+				xdebug_str_addl(str, "...'", 4, 0);
+			}
+			efree(tmp_str);
 		} break;
 
 		case IS_ARRAY:
 			myht = Z_ARRVAL_P(*struc);
 
 			if (!xdebug_zend_hash_is_recursive(myht)) {
-				if (debug_zval) {
-					xdebug_str_add_literal(str, "array (");
-				} else {
-					xdebug_str_addc(str, '[');
-				}
+				xdebug_str_addl(str, "array (", 7, 0);
 				if (level <= options->max_depth) {
 					options->runtime[level].current_element_nr = 0;
 					options->runtime[level].start_element_nr = 0;
@@ -212,11 +205,11 @@ void xdebug_var_export_line(zval **struc, xdebug_str *str, int level, int debug_
 						xdebug_str_chop(str, 2);
 					}
 				} else {
-					xdebug_str_add_literal(str, "...");
+					xdebug_str_addl(str, "...", 3, 0);
 				}
-				xdebug_str_addc(str, debug_zval ? ')' : ']');
+				xdebug_str_addl(str, ")", 1, 0);
 			} else {
-				xdebug_str_add_literal(str, "...");
+				xdebug_str_addl(str, "...", 3, 0);
 			}
 			break;
 
@@ -228,9 +221,8 @@ void xdebug_var_export_line(zval **struc, xdebug_str *str, int level, int debug_
 #endif
 
 			if (!myht || !xdebug_zend_hash_is_recursive(myht)) {
-				xdebug_str_add_literal(str, "class ");
-				xdebug_str_add(str, ZSTR_VAL(Z_OBJCE_P(*struc)->name), 0);
-				xdebug_str_add_literal(str, " { ");
+				char *class_name = (char*) STR_NAME_VAL(Z_OBJCE_P(*struc)->name);
+				xdebug_str_add(str, xdebug_sprintf("class %s { ", class_name), 1);
 
 				if (myht && (level <= options->max_depth)) {
 					options->runtime[level].current_element_nr = 0;
@@ -240,7 +232,7 @@ void xdebug_var_export_line(zval **struc, xdebug_str *str, int level, int debug_
 					xdebug_zend_hash_apply_protection_begin(myht);
 
 					ZEND_HASH_FOREACH_KEY_VAL(myht, num, key, val) {
-						xdebug_object_element_export(*struc, val, num, key, level, str, debug_zval, options, ZSTR_VAL(Z_OBJCE_P(*struc)->name));
+						xdebug_object_element_export(*struc, val, num, key, level, str, debug_zval, options, class_name);
 					} ZEND_HASH_FOREACH_END();
 
 					xdebug_zend_hash_apply_protection_end(myht);
@@ -250,11 +242,11 @@ void xdebug_var_export_line(zval **struc, xdebug_str *str, int level, int debug_
 						xdebug_str_chop(str, 2);
 					}
 				} else {
-					xdebug_str_add_literal(str, "...");
+					xdebug_str_addl(str, "...", 3, 0);
 				}
-				xdebug_str_add_literal(str, " }");
+				xdebug_str_addl(str, " }", 2, 0);
 			} else {
-				xdebug_str_add_literal(str, "...");
+				xdebug_str_addl(str, "...", 3, 0);
 			}
 #if PHP_VERSION_ID >= 70400
 			zend_release_properties(myht);
@@ -267,16 +259,16 @@ void xdebug_var_export_line(zval **struc, xdebug_str *str, int level, int debug_
 			char *type_name;
 
 			type_name = (char *) zend_rsrc_list_get_rsrc_type(Z_RES_P(*struc));
-			xdebug_str_add_fmt(str, "resource(%ld) of type (%s)", Z_RES_P(*struc)->handle, type_name ? type_name : "Unknown");
+			xdebug_str_add(str, xdebug_sprintf("resource(%ld) of type (%s)", Z_RES_P(*struc)->handle, type_name ? type_name : "Unknown"), 1);
 			break;
 		}
 
 		case IS_UNDEF:
-			xdebug_str_add_literal(str, "*uninitialized*");
+			xdebug_str_addl(str, "*uninitialized*", sizeof("*uninitialized*") - 1, 0);
 			break;
 
 		default:
-			xdebug_str_add_literal(str, "NFC");
+			xdebug_str_addl(str, "NFC", 3, 0);
 			break;
 	}
 }
@@ -320,37 +312,36 @@ static void xdebug_var_synopsis(zval **struc, xdebug_str *str, int level, int de
 
 	switch (Z_TYPE_P(*struc)) {
 		case IS_TRUE:
-			xdebug_str_add_literal(str, "true");
+			xdebug_str_addl(str, "true", 4, 0);
 			break;
 
 		case IS_FALSE:
-			xdebug_str_add_literal(str, "false");
+			xdebug_str_addl(str, "false", 5, 0);
 			break;
 
 		case IS_NULL:
-			xdebug_str_add_literal(str, "null");
+			xdebug_str_addl(str, "null", 4, 0);
 			break;
 
 		case IS_LONG:
-			xdebug_str_add_literal(str, "long");
+			xdebug_str_addl(str, "long", 4, 0);
 			break;
 
 		case IS_DOUBLE:
-			xdebug_str_add_literal(str, "double");
+			xdebug_str_addl(str, "double", 6, 0);
 			break;
 
 		case IS_STRING:
-			xdebug_str_add_fmt(str, "string(%d)", Z_STRLEN_P(*struc));
+			xdebug_str_add(str, xdebug_sprintf("string(%d)", Z_STRLEN_P(*struc)), 1);
 			break;
 
 		case IS_ARRAY:
 			myht = Z_ARRVAL_P(*struc);
-			xdebug_str_add_fmt(str, "array(%d)", myht->nNumOfElements);
+			xdebug_str_add(str, xdebug_sprintf("array(%d)", myht->nNumOfElements), 1);
 			break;
 
 		case IS_OBJECT: {
-			xdebug_str_add_literal(str, "class ");
-			xdebug_str_add(str, ZSTR_VAL(Z_OBJCE_P(*struc)->name), 0);
+			xdebug_str_add(str, xdebug_sprintf("class %s", STR_NAME_VAL(Z_OBJCE_P(*struc)->name)), 1);
 			break;
 		}
 
@@ -358,16 +349,16 @@ static void xdebug_var_synopsis(zval **struc, xdebug_str *str, int level, int de
 			char *type_name;
 
 			type_name = (char *) zend_rsrc_list_get_rsrc_type(Z_RES_P(*struc));
-			xdebug_str_add_fmt(str, "resource(%ld) of type (%s)", Z_RES_P(*struc)->handle, type_name ? type_name : "Unknown");
+			xdebug_str_add(str, xdebug_sprintf("resource(%ld) of type (%s)", Z_RES_P(*struc)->handle, type_name ? type_name : "Unknown"), 1);
 			break;
 		}
 
 		case IS_UNDEF:
-			xdebug_str_add_literal(str, "*uninitialized*");
+			xdebug_str_addl(str, "*uninitialized*", sizeof("*uninitialized*") - 1, 0);
 			break;
 
 		default:
-			xdebug_str_add_literal(str, "NFC");
+			xdebug_str_addl(str, "NFC", 3, 0);
 			break;
 
 	}
