@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002-2020 Derick Rethans                               |
+   | Copyright (c) 2002-2021 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 1.01 of the Xdebug license,   |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -12,12 +12,15 @@
    | to obtain it through the world-wide-web, please send a note to       |
    | derick@xdebug.org so we can mail you a copy immediately.             |
    +----------------------------------------------------------------------+
-   | Authors: Derick Rethans <derick@xdebug.org>                          |
-   +----------------------------------------------------------------------+
  */
 
 #include "php.h"
 #include "ext/standard/php_string.h"
+#include "Zend/zend_closures.h"
+#if PHP_VERSION_ID >= 80100
+# include "zend_enum.h"
+#endif
+
 
 #include "php_xdebug.h"
 
@@ -131,6 +134,43 @@ static int xdebug_object_element_export_text_ansi(zval *object, zval *zv_nptr, z
 	return 0;
 }
 
+static void handle_closure(xdebug_str *str, zval *obj, int level, int mode)
+{
+	const zend_function *closure_function;
+
+	if (Z_TYPE_P(obj) != IS_OBJECT || !instanceof_function(Z_OBJCE_P(obj), zend_ce_closure)) {
+		return;
+	}
+
+#if PHP_VERSION_ID >= 80000
+	closure_function = zend_get_closure_method_def(Z_OBJ_P(obj));
+#else
+	closure_function = zend_get_closure_method_def(obj);
+#endif
+
+	xdebug_str_add_fmt(
+		str, "%*s%s%svirtual%s $closure =>\n%*s\"",
+		(level * 4) - 2, "",
+		ANSI_COLOR_MODIFIER, ANSI_COLOR_BOLD, ANSI_COLOR_RESET,
+		(level * 4) - 2, ""
+	);
+
+	if (closure_function->common.scope) {
+		if (closure_function->common.fn_flags & ZEND_ACC_STATIC) {
+			xdebug_str_add_fmt(str, "%s", ANSI_COLOR_OBJECT);
+			xdebug_str_add_zstr(str, closure_function->common.scope->name);
+			xdebug_str_add_fmt(str, "%s::", ANSI_COLOR_RESET);
+		} else {
+			xdebug_str_add_fmt(str, "%s$this%s->", ANSI_COLOR_OBJECT, ANSI_COLOR_RESET);
+		}
+	}
+
+	xdebug_str_add_fmt(str, "%s", ANSI_COLOR_STRING);
+	xdebug_str_add_zstr(str, closure_function->common.function_name);
+	xdebug_str_add_fmt(str, "%s\"\n", ANSI_COLOR_RESET);
+}
+
+
 static void xdebug_var_export_text_ansi(zval **struc, xdebug_str *str, int mode, int level, int debug_zval, xdebug_var_export_options *options)
 {
 	HashTable *myht;
@@ -253,11 +293,45 @@ static void xdebug_var_export_text_ansi(zval **struc, xdebug_str *str, int mode,
 			}
 			break;
 
-		case IS_OBJECT:
+		case IS_OBJECT: {
+#if PHP_VERSION_ID >= 80100
+			zend_class_entry *ce = Z_OBJCE_P(*struc);
+			if (ce->ce_flags & ZEND_ACC_ENUM) {
+				zval *case_name_zval = zend_enum_fetch_case_name(Z_OBJ_P(*struc));
+				xdebug_str_add_fmt(
+					str, "%senum%s %s%s%s::%s%s%s",
+					ANSI_COLOR_BOLD, ANSI_COLOR_BOLD_OFF,
+					ANSI_COLOR_OBJECT, ZSTR_VAL(ce->name), ANSI_COLOR_RESET,
+					ANSI_COLOR_STRING, Z_STRVAL_P(case_name_zval), ANSI_COLOR_RESET
+				);
+				if (ce->enum_backing_type != IS_UNDEF) {
+					zval *value = zend_enum_fetch_case_value(Z_OBJ_P(*struc));
+
+					if (ce->enum_backing_type == IS_LONG) {
+						xdebug_str_add_fmt(
+							str, " : %s%s%s(%s%d%s)",
+							ANSI_COLOR_BOLD, "int", ANSI_COLOR_RESET,
+							ANSI_COLOR_LONG, Z_LVAL_P(value), ANSI_COLOR_RESET
+						);
+					}
+
+					if (ce->enum_backing_type == IS_STRING) {
+						xdebug_str_add_fmt(
+							str, " : %s%s%s(%s\"%s\"%s)",
+							ANSI_COLOR_BOLD, "string", ANSI_COLOR_RESET,
+							ANSI_COLOR_STRING, Z_STRVAL_P(value), ANSI_COLOR_RESET
+						);
+					}
+				}
+				xdebug_str_addc(str, ';');
+				break;
+			}
+#endif
+
 #if PHP_VERSION_ID >= 70400
-			myht = xdebug_objdebug_pp(struc);
+			myht = xdebug_objdebug_pp(struc, XDEBUG_VAR_OBJDEBUG_USE_DEBUGINFO);
 #else
-			myht = xdebug_objdebug_pp(struc, &is_temp);
+			myht = xdebug_objdebug_pp(struc, &is_temp, XDEBUG_VAR_OBJDEBUG_USE_DEBUGINFO);
 #endif
 
 			if (!myht || !xdebug_zend_hash_is_recursive(myht)) {
@@ -269,6 +343,8 @@ static void xdebug_var_export_text_ansi(zval **struc, xdebug_str *str, int mode,
 					ANSI_COLOR_LONG, myht ? myht->nNumOfElements : 0, ANSI_COLOR_RESET
 				);
 
+				handle_closure(str, *struc, level, mode);
+
 				if (myht && (level <= options->max_depth)) {
 					options->runtime[level].current_element_nr = 0;
 					options->runtime[level].start_element_nr = 0;
@@ -294,6 +370,7 @@ static void xdebug_var_export_text_ansi(zval **struc, xdebug_str *str, int mode,
 			xdebug_var_maybe_destroy_ht(myht, is_temp);
 #endif
 			break;
+		}
 
 		case IS_RESOURCE: {
 			char *type_name;
