@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002-2019 Derick Rethans                               |
+   | Copyright (c) 2002-2021 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 1.01 of the Xdebug license,   |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -12,11 +12,10 @@
    | to obtain it through the world-wide-web, please send a note to       |
    | derick@xdebug.org so we can mail you a copy immediately.             |
    +----------------------------------------------------------------------+
-   | Authors: Derick Rethans <derick@xdebug.org>                          |
-   +----------------------------------------------------------------------+
  */
 
 #include "var_export_xml.h"
+#include "Zend/zend_closures.h"
 
 static xdebug_str *prepare_variable_name(xdebug_str *name)
 {
@@ -52,7 +51,7 @@ static int encoding_requested(char *value, size_t value_len)
 	return 0;
 }
 
-static void check_if_extended_properies_are_needed(xdebug_var_export_options *options, xdebug_str *name, xdebug_str *fullname, zval *value)
+static void check_if_extended_properties_are_needed(xdebug_var_export_options *options, xdebug_str *name, xdebug_str *fullname, zval *value)
 {
 	if (!options->extended_properties || options->encode_as_extended_property) {
 		return;
@@ -130,8 +129,10 @@ void xdebug_var_xml_attach_uninitialized_var(xdebug_var_export_options *options,
 ** XML node printing routines
 */
 
-#define XDEBUG_OBJECT_ITEM_TYPE_PROPERTY        1
-#define XDEBUG_OBJECT_ITEM_TYPE_STATIC_PROPERTY 2
+#define XDEBUG_OBJECT_ITEM_TYPE_PROPERTY           0
+#define XDEBUG_OBJECT_ITEM_TYPE_STATIC_PROPERTY 1<<0
+#define XDEBUG_OBJECT_ITEM_TYPE_READONLY        1<<1
+
 
 typedef struct
 {
@@ -142,6 +143,12 @@ typedef struct
 	zval         *zv;
 } xdebug_object_item;
 
+static void merged_hash_object_item_dtor(zval *data)
+{
+	 xdebug_object_item *item = Z_PTR_P(data);
+
+	 xdfree(item);
+}
 
 static int object_item_add_to_merged_hash(zval *zv_nptr, zend_ulong index_key, zend_string *hash_key, HashTable *merged, int object_type)
 {
@@ -257,9 +264,9 @@ static int xdebug_array_element_export_xml_node(zval *zv_nptr, zend_ulong index_
 
 			if (parent_name) {
 				xdebug_str_add_str(&full_name, parent_name);
-				xdebug_str_addl(&full_name, "[\"", 2, 0);
+				xdebug_str_add_literal(&full_name, "[\"");
 				xdebug_str_addl(&full_name, tmp_fullname_zstr->val, tmp_fullname_zstr->len, 0);
-				xdebug_str_addl(&full_name, "\"]", 2, 0);
+				xdebug_str_add_literal(&full_name, "\"]");
 			}
 
 			zend_string_release(tmp_fullname_zstr);
@@ -278,7 +285,7 @@ static int xdebug_array_element_export_xml_node(zval *zv_nptr, zend_ulong index_
 			xdfree(tmp_idx);
 		}
 
-		check_if_extended_properies_are_needed(options, name, full_name.l ? &full_name : NULL, *zv);
+		check_if_extended_properties_are_needed(options, name, full_name.l ? &full_name : NULL, *zv);
 		add_xml_attribute_or_element(options, node, "name", 4, name);
 		if (full_name.l) {
 			add_xml_attribute_or_element(options, node, "fullname", 8, &full_name);
@@ -330,12 +337,16 @@ static int xdebug_object_element_export_xml_node(xdebug_object_item *item_nptr,
 				tmp_fullname = xdebug_str_new();
 
 				xdebug_str_add_str(tmp_fullname, parent_name);
-				xdebug_str_add(tmp_fullname, (*item)->type == XDEBUG_OBJECT_ITEM_TYPE_STATIC_PROPERTY ? "::" : "->", 0);
+				if ((*item)->type & XDEBUG_OBJECT_ITEM_TYPE_STATIC_PROPERTY) {
+					xdebug_str_add_literal(tmp_fullname, "::");
+				} else {
+					xdebug_str_add_literal(tmp_fullname, "->");
+				}
 
 				/* Only in dynamic and *public* properties can we have non-standard characters */
 				if (strcmp(modifier, "private") != 0 || strcmp(class_name, prop_class_name) == 0) {
 					if (property_name->l == 0) {
-						xdebug_str_addl(tmp_fullname, "{\"\"}", 4, 0);
+						xdebug_str_add_literal(tmp_fullname, "{\"\"}");
 					} else {
 						if (memchr(property_name->d, '-', property_name->l) == NULL && memchr(property_name->d, '[', property_name->l) == NULL && memchr(property_name->d, '{', property_name->l) == NULL) {
 							xdebug_str_add_str(tmp_fullname, property_name);
@@ -345,9 +356,9 @@ static int xdebug_object_element_export_xml_node(xdebug_object_item *item_nptr,
 
 							tmp_slashed_string = xdebug_addslashes(tmp_string);
 
-							xdebug_str_addl(tmp_fullname, "{\"", 2, 0);
-							xdebug_str_addl(tmp_fullname, tmp_slashed_string->val, tmp_slashed_string->len, 0);
-							xdebug_str_addl(tmp_fullname, "\"}", 2, 0);
+							xdebug_str_add_literal(tmp_fullname, "{\"");
+							xdebug_str_add_zstr(tmp_fullname, tmp_slashed_string);
+							xdebug_str_add_literal(tmp_fullname, "\"}");
 
 							zend_string_release(tmp_slashed_string);
 							zend_string_release(tmp_string);
@@ -380,20 +391,29 @@ static int xdebug_object_element_export_xml_node(xdebug_object_item *item_nptr,
 			if (parent_name) {
 				char       *tmp_formatted_prop;
 
-				tmp_formatted_prop = xdebug_sprintf("%s%s" XDEBUG_INT_FMT, parent_name, (*item)->type == XDEBUG_OBJECT_ITEM_TYPE_STATIC_PROPERTY ? "::" : "->", (*item)->index_key);
+				tmp_formatted_prop = xdebug_sprintf("%s%s" XDEBUG_INT_FMT, parent_name, (*item)->type & XDEBUG_OBJECT_ITEM_TYPE_STATIC_PROPERTY ? "::" : "->", (*item)->index_key);
 				tmp_fullname = xdebug_str_create_from_char(tmp_formatted_prop);
 
 				xdfree(tmp_formatted_prop);
 			}
 		}
 
-		check_if_extended_properies_are_needed(options, tmp_name, tmp_fullname, (*item)->zv);
+		check_if_extended_properties_are_needed(options, tmp_name, tmp_fullname, (*item)->zv);
 		add_xml_attribute_or_element(options, node, "name", 4, tmp_name);
 		if (tmp_fullname) {
 			add_xml_attribute_or_element(options, node, "fullname", 8, tmp_fullname);
 		}
 
-		xdebug_xml_add_attribute_ex(node, "facet", xdebug_sprintf("%s%s", (*item)->type == XDEBUG_OBJECT_ITEM_TYPE_STATIC_PROPERTY ? "static " : "", modifier), 0, 1);
+
+		xdebug_xml_add_attribute_ex(
+			node, "facet",
+			xdebug_sprintf("%s%s%s",
+				(*item)->type & XDEBUG_OBJECT_ITEM_TYPE_STATIC_PROPERTY ? "static " : "",
+				modifier,
+				(*item)->type & XDEBUG_OBJECT_ITEM_TYPE_READONLY ? " readonly" : ""
+			),
+			0, 1
+		);
 		xdebug_xml_add_child(parent, node);
 		xdebug_var_export_xml_node(&((*item)->zv), tmp_fullname ? tmp_fullname : NULL, node, options, level + 1);
 
@@ -465,6 +485,12 @@ void xdebug_var_xml_attach_static_vars(xdebug_xml_node *node, xdebug_var_export_
 
 	xdebug_zend_hash_apply_protection_begin(static_members);
 
+#if PHP_VERSION_ID >= 80100
+	if (ce->default_static_members_count && !CE_STATIC_MEMBERS(ce)) {
+		zend_class_init_statics(ce);
+	}
+#endif
+
 	ZEND_HASH_FOREACH_PTR(static_members, zpi) {
 		xdebug_var_xml_attach_property_with_contents(zpi, static_container, options, ce, STR_NAME_VAL(ce->name), &children);
 	} ZEND_HASH_FOREACH_END();
@@ -565,23 +591,26 @@ void xdebug_var_export_xml_node(zval **struc, xdebug_str *name, xdebug_xml_node
 
 		case IS_OBJECT: {
 			HashTable          *merged_hash;
-			xdebug_str         *class_name;
+			zend_string        *class_name;
 			zend_class_entry   *ce;
-#if PHP_VERSION_ID < 70400 
+#if PHP_VERSION_ID < 70400
 			int                 is_temp;
 #endif
+			int                 extra_children = 0;
 			zend_property_info *zpi_val;
 
 			ALLOC_HASHTABLE(merged_hash);
-			zend_hash_init(merged_hash, 128, NULL, NULL, 0);
+			zend_hash_init(merged_hash, 128, NULL, merged_hash_object_item_dtor, 0);
 
-			class_name = xdebug_str_create(STR_NAME_VAL(Z_OBJCE_P(*struc)->name), STR_NAME_LEN(Z_OBJCE_P(*struc)->name));
-			ce = xdebug_fetch_class(class_name->d, class_name->l, ZEND_FETCH_CLASS_DEFAULT);
+			class_name = Z_OBJCE_P(*struc)->name;
+			ce = zend_fetch_class(class_name, ZEND_FETCH_CLASS_DEFAULT);
 
 			/* Adding static properties */
 			xdebug_zend_hash_apply_protection_begin(&ce->properties_info);
 
-#if PHP_VERSION_ID >= 70400
+#if PHP_VERSION_ID >= 80100
+			zend_class_init_statics(ce);
+#elif PHP_VERSION_ID >= 70400
 			if (ce->type == ZEND_INTERNAL_CLASS || (ce->ce_flags & ZEND_ACC_IMMUTABLE)) {
 				zend_class_init_statics(ce);
 			}
@@ -595,9 +624,9 @@ void xdebug_var_export_xml_node(zval **struc, xdebug_str *name, xdebug_xml_node
 
 			/* Adding normal properties */
 #if PHP_VERSION_ID >= 70400
-			myht = xdebug_objdebug_pp(struc);
+			myht = xdebug_objdebug_pp(struc, XDEBUG_VAR_OBJDEBUG_DEFAULT);
 #else
-			myht = xdebug_objdebug_pp(struc, &is_temp);
+			myht = xdebug_objdebug_pp(struc, &is_temp, XDEBUG_VAR_OBJDEBUG_DEFAULT);
 #endif
 			if (myht) {
 				zval *tmp_val;
@@ -605,19 +634,116 @@ void xdebug_var_export_xml_node(zval **struc, xdebug_str *name, xdebug_xml_node
 				xdebug_zend_hash_apply_protection_begin(myht);
 
 				ZEND_HASH_FOREACH_KEY_VAL_IND(myht, num, key, tmp_val) {
-					object_item_add_to_merged_hash(tmp_val, num, key, merged_hash, (int) XDEBUG_OBJECT_ITEM_TYPE_PROPERTY);
+					int flags = XDEBUG_OBJECT_ITEM_TYPE_PROPERTY;
+
+#if PHP_VERSION_ID >= 80100
+					if (ce->type != ZEND_INTERNAL_CLASS) {
+						const char         *cls_name, *tmp_prop_name;
+						size_t              tmp_prop_name_len;
+						zend_string        *unmangled;
+						zend_property_info *info;
+
+						zend_unmangle_property_name_ex(key, &cls_name, &tmp_prop_name, &tmp_prop_name_len);
+						unmangled = zend_string_init_interned(tmp_prop_name, tmp_prop_name_len, 0);
+
+						info = zend_get_property_info(Z_OBJCE_P(*struc), unmangled, 1);
+
+						zend_string_release(unmangled);
+
+						if (info && info != ZEND_WRONG_PROPERTY_INFO && info->flags & ZEND_ACC_READONLY) {
+							flags |= XDEBUG_OBJECT_ITEM_TYPE_READONLY;
+						}
+					}
+#endif
+					object_item_add_to_merged_hash(tmp_val, num, key, merged_hash, flags);
 				} ZEND_HASH_FOREACH_END();
 
 				xdebug_zend_hash_apply_protection_end(myht);
 			}
 
 			xdebug_xml_add_attribute(node, "type", "object");
-			add_xml_attribute_or_element(options, node, "classname", 9, class_name);
+
+#if PHP_VERSION_ID >= 80100 // Enums
+			{
+				zend_class_entry *ce = Z_OBJCE_P(*struc);
+				if (ce->ce_flags & ZEND_ACC_ENUM) {
+					xdebug_xml_expand_attribute_value(node, "facet", "enum");
+				}
+			}
+#endif
+
+			if (instanceof_function(Z_OBJCE_P(*struc), zend_ce_closure)) {
+				xdebug_xml_node *closure_cont, *closure_func;
+#if PHP_VERSION_ID >= 80000
+				const zend_function *closure_function = zend_get_closure_method_def(Z_OBJ_P(*struc));
+#else
+				const zend_function *closure_function = zend_get_closure_method_def(*struc);
+#endif
+
+				xdebug_xml_expand_attribute_value(node, "facet", "closure");
+
+				closure_cont = xdebug_xml_node_init("property");
+				xdebug_xml_add_attribute(closure_cont, "facet", "virtual readonly");
+				xdebug_xml_add_attribute(closure_cont, "name", "{closure}");
+				xdebug_xml_add_attribute(closure_cont, "type", "array");
+				xdebug_xml_add_attribute(closure_cont, "children", "1");
+				xdebug_xml_add_attribute(closure_cont, "page", "0");
+				xdebug_xml_add_attribute(closure_cont, "pagesize", "2");
+
+				if (closure_function->common.scope) {
+					xdebug_xml_node *closure_scope = xdebug_xml_node_init("property");
+					xdebug_xml_add_attribute(closure_scope, "facet", "readonly");
+					xdebug_xml_add_attribute(closure_scope, "name", "scope");
+					xdebug_xml_add_attribute(closure_scope, "type", "string");
+
+					if (closure_function->common.fn_flags & ZEND_ACC_STATIC) {
+						xdebug_xml_add_text_ex(
+							closure_scope,
+							ZSTR_VAL(closure_function->common.scope->name),
+							ZSTR_LEN(closure_function->common.scope->name),
+							0, 0
+						);
+					} else {
+						xdebug_xml_add_text_ex(closure_scope, (char*) "$this", 6, 0, 0);
+					}
+
+					xdebug_xml_add_child(closure_cont, closure_scope);
+					xdebug_xml_add_attribute(closure_cont, "numchildren", "2");
+				} else {
+					xdebug_xml_add_attribute(closure_cont, "numchildren", "1");
+				}
+
+				closure_func = xdebug_xml_node_init("property");
+				xdebug_xml_add_attribute(closure_func, "facet", "readonly");
+				xdebug_xml_add_attribute(closure_func, "name", "function");
+				xdebug_xml_add_attribute(closure_func, "type", "string");
+				xdebug_xml_add_text_ex(
+					closure_func,
+					ZSTR_VAL(closure_function->common.function_name),
+					ZSTR_LEN(closure_function->common.function_name),
+					0, 0
+				);
+				xdebug_xml_add_child(closure_cont, closure_func);
+
+				xdebug_xml_add_child(node, closure_cont);
+				extra_children = 1;
+			}
+
+			{
+				xdebug_str tmp_str;
+				tmp_str.d = ZSTR_VAL(class_name);
+				tmp_str.l = ZSTR_LEN(class_name);
+				add_xml_attribute_or_element(options, node, "classname", 9, &tmp_str);
+			}
 			xdebug_xml_add_attribute(node, "children", merged_hash->nNumOfElements ? "1" : "0");
 
 
-			if (!xdebug_zend_hash_is_recursive(myht)) {
-				xdebug_xml_add_attribute_ex(node, "numchildren", xdebug_sprintf("%d", zend_hash_num_elements(merged_hash)), 0, 1);
+			if (!myht || !xdebug_zend_hash_is_recursive(myht)) {
+				xdebug_xml_add_attribute_ex(
+					node, "numchildren",
+					xdebug_sprintf("%d", zend_hash_num_elements(merged_hash) + extra_children),
+					0, 1
+				);
 				if (level < options->max_depth) {
 					xdebug_xml_add_attribute_ex(node, "page", xdebug_sprintf("%d", options->runtime[level].page), 0, 1);
 					xdebug_xml_add_attribute_ex(node, "pagesize", xdebug_sprintf("%d", options->max_children), 0, 1);
@@ -633,7 +759,7 @@ void xdebug_var_export_xml_node(zval **struc, xdebug_str *name, xdebug_xml_node
 					xdebug_zend_hash_apply_protection_begin(merged_hash);
 
 					ZEND_HASH_FOREACH_KEY_PTR(merged_hash, num, key, xoi_val) {
-						xdebug_object_element_export_xml_node(xoi_val, level, node, name, options, class_name->d);
+						xdebug_object_element_export_xml_node(xoi_val, level, node, name, options, ZSTR_VAL(class_name));
 					} ZEND_HASH_FOREACH_END();
 
 					xdebug_zend_hash_apply_protection_end(merged_hash);
@@ -642,7 +768,6 @@ void xdebug_var_export_xml_node(zval **struc, xdebug_str *name, xdebug_xml_node
 
 			zend_hash_destroy(merged_hash);
 			FREE_HASHTABLE(merged_hash);
-			xdebug_str_free(class_name);
 #if PHP_VERSION_ID >= 70400
 			zend_release_properties(myht);
 #else
@@ -689,7 +814,7 @@ xdebug_xml_node* xdebug_get_zval_value_xml_node_ex(xdebug_str *name, zval *val,
 			case XDEBUG_VAR_TYPE_STATIC: {
 				xdebug_str tmp_formatted_name = XDEBUG_STR_INITIALIZER;
 
-				xdebug_str_addl(&tmp_formatted_name, "::", 2, 0);
+				xdebug_str_add_literal(&tmp_formatted_name, "::");
 				xdebug_str_add_str(&tmp_formatted_name, name);
 
 				short_name = xdebug_str_copy(&tmp_formatted_name);
@@ -704,7 +829,7 @@ xdebug_xml_node* xdebug_get_zval_value_xml_node_ex(xdebug_str *name, zval *val,
 				break;
 		}
 
-		check_if_extended_properies_are_needed(options, short_name, full_name, val);
+		check_if_extended_properties_are_needed(options, short_name, full_name, val);
 		add_xml_attribute_or_element(options, node, "name", 4, short_name);
 		add_xml_attribute_or_element(options, node, "fullname", 8, full_name);
 	}
