@@ -13,7 +13,6 @@
    | derick@xdebug.org so we can mail you a copy immediately.             |
    +----------------------------------------------------------------------+
    | Authors: Derick Rethans <derick@xdebug.org>                          |
-   |          Michael Voříšek <mvorisek@mvorisek.cz>                      |
    +----------------------------------------------------------------------+
  */
 
@@ -30,7 +29,6 @@
 #include "profiler.h"
 #include "profiler_private.h"
 
-#include "lib/log.h"
 #include "lib/mm.h"
 #include "lib/str.h"
 #include "lib/var.h"
@@ -38,17 +36,13 @@
 
 ZEND_EXTERN_MODULE_GLOBALS(xdebug)
 
-int xdebug_profiler_exit_handler(XDEBUG_OPCODE_HANDLER_ARGS);
-
 void xdebug_init_profiler_globals(xdebug_profiler_globals_t *xg)
 {
-	xg->active = 0;
+	xg->profiler_enabled = 0;
 }
 
 void xdebug_profiler_minit(void)
 {
-	/* Overload the "exit" opcode */
-	xdebug_set_opcode_handler(ZEND_EXIT, xdebug_profiler_exit_handler);
 }
 
 void xdebug_profiler_mshutdown(void)
@@ -62,90 +56,74 @@ void xdebug_profiler_rinit(void)
 	XG_PROF(profile_filename_refs) = NULL;
 	XG_PROF(profile_functionname_refs) = NULL;
 	XG_PROF(profile_last_filename_ref) = 0;
-	XG_PROF(php_internal_seen_before) = 0;
 	XG_PROF(profile_last_functionname_ref) = 0;
-	XG_PROF(active) = 0;
-}
-
-static void deinit_if_active(void)
-{
-	if (!XG_PROF(active)) {
-		return;
-	}
-
-	xdebug_profiler_deinit();
+	XG_PROF(profiler_enabled) = 0;
 }
 
 void xdebug_profiler_post_deactivate(void)
 {
-	deinit_if_active();
+	if (XG_PROF(profiler_enabled)) {
+		xdebug_profiler_deinit();
+	}
 }
 
 void xdebug_profiler_pcntl_exec_handler(void)
 {
-	deinit_if_active();
+	if (XG_PROF(profiler_enabled)) {
+		xdebug_profiler_deinit();
+	}
 }
 
-int xdebug_profiler_exit_handler(XDEBUG_OPCODE_HANDLER_ARGS)
+void xdebug_profiler_exit_handler(void)
 {
-	const zend_op *cur_opcode = execute_data->opline;
-
-	deinit_if_active();
-
-	return xdebug_call_original_opcode_handler_if_set(cur_opcode->opcode, XDEBUG_OPCODE_HANDLER_ARGS_PASSTHRU);
+	if (XG_PROF(profiler_enabled)) {
+		xdebug_profiler_deinit();
+	}
 }
 
 
 void xdebug_profiler_init_if_requested(zend_op_array *op_array)
 {
-	if (XG_PROF(active)) {
-		return;
-	}
-
-	if (EG(flags) & EG_FLAGS_IN_SHUTDOWN) {
-		return;
-	}
-
-	if (xdebug_lib_start_with_request() || xdebug_lib_start_with_trigger(XDEBUG_MODE_PROFILING, NULL)) {
+	/* Check for special GET/POST parameter to start profiling */
+	if (
+		!XG_PROF(profiler_enabled) &&
+		(XINI_PROF(profiler_enable) || xdebug_trigger_enabled(XINI_PROF(profiler_enable_trigger), "XDEBUG_PROFILE", XINI_PROF(profiler_enable_trigger_value)))
+	) {
 		xdebug_profiler_init((char*) STR_NAME_VAL(op_array->filename));
 	}
 }
 
 void xdebug_profiler_execute_ex(function_stack_entry *fse, zend_op_array *op_array)
 {
-	if (!XG_PROF(active)) {
-		return;
+	if (XG_PROF(profiler_enabled)) {
+		/* Calculate all elements for profile entries */
+		xdebug_profiler_add_function_details_user(fse, op_array);
+		xdebug_profiler_function_begin(fse);
 	}
-
-	/* Calculate all elements for profile entries */
-	xdebug_profiler_add_function_details_user(fse, op_array);
-	xdebug_profiler_function_begin(fse);
 }
 
 void xdebug_profiler_execute_ex_end(function_stack_entry *fse)
 {
-	xdebug_profiler_function_end(fse);
-	xdebug_profiler_free_function_details(fse);
+	if (XG_PROF(profiler_enabled)) {
+		xdebug_profiler_function_end(fse);
+		xdebug_profiler_free_function_details(fse);
+	}
 }
 
 void xdebug_profiler_execute_internal(function_stack_entry *fse)
 {
-	if (!XG_PROF(active)) {
-		return;
+	if (XG_PROF(profiler_enabled)) {
+		xdebug_profiler_add_function_details_internal(fse);
+		xdebug_profiler_function_begin(fse);
 	}
-
-	xdebug_profiler_add_function_details_internal(fse);
-	xdebug_profiler_function_begin(fse);
 }
 
 void xdebug_profiler_execute_internal_end(function_stack_entry *fse)
 {
-	if (!XG_PROF(active)) {
-		return;
+	if (XG_PROF(profiler_enabled)) {
+		xdebug_profiler_function_end(fse);
+		xdebug_profiler_free_function_details(fse);
 	}
-
-	xdebug_profiler_function_end(fse);
-	xdebug_profiler_free_function_details(fse);
 }
 
 void xdebug_profile_call_entry_dtor(void *dummy, void *elem)
@@ -156,7 +134,7 @@ void xdebug_profile_call_entry_dtor(void *dummy, void *elem)
 		xdfree(ce->function);
 	}
 	if (ce->filename) {
-		zend_string_release(ce->filename);
+		xdfree(ce->filename);
 	}
 	xdfree(ce);
 }
@@ -168,18 +146,15 @@ static void profiler_write_header(FILE *file, char *script_name)
 	}
 	fprintf(file, "version: 1\ncreator: xdebug %s (PHP %s)\n", XDEBUG_VERSION, PHP_VERSION);
 	fprintf(file, "cmd: %s\npart: 1\npositions: line\n\n", script_name);
-	fprintf(file, "events: Time_(10ns) Memory_(bytes)\n\n");
+	fprintf(file, "events: Time Memory\n\n");
 	fflush(file);
 }
 
-#define NANOTIME_SCALE_10NS(nanotime) ((unsigned long)(((nanotime) + 5) / 10))
-
 void xdebug_profiler_init(char *script_name)
 {
 	char *filename = NULL, *fname = NULL;
-	char *output_dir = NULL;
 
-	if (XG_PROF(active)) {
+	if (XG_PROF(profiler_enabled)) {
 		return;
 	}
 
@@ -190,24 +165,23 @@ void xdebug_profiler_init(char *script_name)
 		return;
 	}
 
-	/* Add a slash if none is present in the output_dir setting */
-	output_dir = xdebug_lib_get_output_dir(); /* not duplicated */
-
-	if (IS_SLASH(output_dir[strlen(output_dir) - 1])) {
-		filename = xdebug_sprintf("%s%s", output_dir, fname);
+	/* Add a slash if none is present in the profiler_output_dir setting */
+	if (IS_SLASH(XINI_PROF(profiler_output_dir)[strlen(XINI_PROF(profiler_output_dir)) - 1])) {
+		filename = xdebug_sprintf("%s%s", XINI_PROF(profiler_output_dir), fname);
 	} else {
-		filename = xdebug_sprintf("%s%c%s", output_dir, DEFAULT_SLASH, fname);
+		filename = xdebug_sprintf("%s%c%s", XINI_PROF(profiler_output_dir), DEFAULT_SLASH, fname);
 	}
+	xdfree(fname);
 
 	if (XINI_PROF(profiler_append)) {
 		XG_PROF(profile_file) = xdebug_fopen(filename, "a", NULL, &XG_PROF(profile_filename));
 	} else {
 		XG_PROF(profile_file) = xdebug_fopen(filename, "w", NULL, &XG_PROF(profile_filename));
 	}
+	xdfree(filename);
 
 	if (!XG_PROF(profile_file)) {
-		xdebug_log_diagnose_permissions(XLOG_CHAN_PROFILE, output_dir, fname);
-		goto return_and_free_names;
+		return;
 	}
 
 	profiler_write_header(XG_PROF(profile_file), script_name);
@@ -218,39 +192,37 @@ void xdebug_profiler_init(char *script_name)
 		ctr.line = xdebug_sprintf("X-Xdebug-Profile-Filename: %s", XG_PROF(profile_filename));
 		ctr.line_len = strlen(ctr.line);
 		sapi_header_op(SAPI_HEADER_REPLACE, &ctr);
-		xdfree((void*) ctr.line);
+		xdfree(ctr.line);
 	}
 
-	XG_PROF(profiler_start_nanotime) = xdebug_get_nanotime();
+	XG_PROF(profiler_start_time) = xdebug_get_utime();
 
-	XG_PROF(active) = 1;
-	XG_PROF(profile_filename_refs) = xdebug_hash_alloc(128, xdfree);
-	XG_PROF(profile_functionname_refs) = xdebug_hash_alloc(128, xdfree);
-	XG_PROF(profile_last_filename_ref) = 1;
+	XG_PROF(profiler_enabled) = 1;
+	XG_PROF(profile_filename_refs) = xdebug_hash_alloc(128, NULL);
+	XG_PROF(profile_functionname_refs) = xdebug_hash_alloc(128, NULL);
+	XG_PROF(profile_last_filename_ref) = 0;
 	XG_PROF(profile_last_functionname_ref) = 0;
-
-return_and_free_names:
-	xdfree(filename);
-	xdfree(fname);
+	return;
 }
 
 void xdebug_profiler_deinit()
 {
-	function_stack_entry *fse = XDEBUG_VECTOR_TAIL(XG_BASE(stack));
-	int                   i;
+	function_stack_entry *fse;
+	xdebug_llist_element *le;
 
-	for (i = 0; i < XDEBUG_VECTOR_COUNT(XG_BASE(stack)); i++, fse--) {
+	for (le = XDEBUG_LLIST_TAIL(XG_BASE(stack)); le != NULL; le = XDEBUG_LLIST_PREV(le)) {
+		fse = XDEBUG_LLIST_VALP(le);
 		xdebug_profiler_function_end(fse);
 	}
 
 	fprintf(
 		XG_PROF(profile_file),
 		"summary: %lu %zd\n\n",
-		NANOTIME_SCALE_10NS(xdebug_get_nanotime() - XG_PROF(profiler_start_nanotime)),
+		(unsigned long) ((xdebug_get_utime() - (XG_PROF(profiler_start_time))) * 1000000),
 		zend_memory_peak_usage(0)
 	);
 
-	XG_PROF(active) = 0;
+	XG_PROF(profiler_enabled) = 0;
 
 	fflush(XG_PROF(profile_file));
 
@@ -271,15 +243,17 @@ void xdebug_profiler_deinit()
 
 static inline void xdebug_profiler_function_push(function_stack_entry *fse)
 {
-	fse->profile.nanotime += (xdebug_get_nanotime() - fse->profile.nanotime_mark);
-	fse->profile.nanotime_mark = 0;
-	fse->profile.memory += (zend_memory_usage(0) - fse->profile.mem_mark);
+	fse->profile.time += xdebug_get_utime();
+	fse->profile.time -= fse->profile.mark;
+	fse->profile.mark = 0;
+	fse->profile.memory += zend_memory_usage(0);
+	fse->profile.memory -= fse->profile.mem_mark;
 	fse->profile.mem_mark = 0;
 }
 
 void xdebug_profiler_function_continue(function_stack_entry *fse)
 {
-	fse->profile.nanotime_mark = xdebug_get_nanotime();
+	fse->profile.mark = xdebug_get_utime();
 }
 
 void xdebug_profiler_function_pause(function_stack_entry *fse)
@@ -287,39 +261,29 @@ void xdebug_profiler_function_pause(function_stack_entry *fse)
 	xdebug_profiler_function_push(fse);
 }
 
-static inline void add_filename_ref(xdebug_str *buffer, char *name)
+static char* get_filename_ref(char *name)
 {
-	char *ref;
+	long nr;
 
-	if (xdebug_hash_find(XG_PROF(profile_filename_refs), name, strlen(name), (void*) &ref)) {
-		xdebug_str_add(buffer, ref, 0);
+	if (xdebug_hash_find(XG_PROF(profile_filename_refs), name, strlen(name), (void*) &nr)) {
+		return xdebug_sprintf("(%d)", nr);
 	} else {
 		XG_PROF(profile_last_filename_ref)++;
-		ref = xdebug_sprintf("(%d)", XG_PROF(profile_last_filename_ref));
-
-		xdebug_hash_add(XG_PROF(profile_filename_refs), name, strlen(name), (void*) ref);
-
-		xdebug_str_add(buffer, ref, 0);
-		xdebug_str_addc(buffer, ' ');
-		xdebug_str_add(buffer, name,  0);
+		xdebug_hash_add(XG_PROF(profile_filename_refs), name, strlen(name), (void*) (size_t) XG_PROF(profile_last_filename_ref));
+		return xdebug_sprintf("(%d) %s", XG_PROF(profile_last_filename_ref), name);
 	}
 }
 
-static inline void add_functionname_ref(xdebug_str *buffer, char *name)
+static char* get_functionname_ref(char *name)
 {
-	char *ref;
+	long nr;
 
-	if (xdebug_hash_find(XG_PROF(profile_functionname_refs), name, strlen(name), (void*) &ref)) {
-		xdebug_str_add(buffer, ref, 0);
+	if (xdebug_hash_find(XG_PROF(profile_functionname_refs), name, strlen(name), (void*) &nr)) {
+		return xdebug_sprintf("(%d)", nr);
 	} else {
 		XG_PROF(profile_last_functionname_ref)++;
-		ref = xdebug_sprintf("(%d)", XG_PROF(profile_last_functionname_ref));
-
-		xdebug_hash_add(XG_PROF(profile_functionname_refs), name, strlen(name), (void*) ref);
-
-		xdebug_str_add(buffer, ref, 0);
-		xdebug_str_addc(buffer, ' ');
-		xdebug_str_add(buffer, name,  0);
+		xdebug_hash_add(XG_PROF(profile_functionname_refs), name, strlen(name), (void*) (size_t) XG_PROF(profile_last_functionname_ref));
+		return xdebug_sprintf("(%d) %s", XG_PROF(profile_last_functionname_ref), name);
 	}
 }
 
@@ -333,7 +297,7 @@ void xdebug_profiler_add_function_details_user(function_stack_entry *fse, zend_o
 		case XFUNC_INCLUDE_ONCE:
 		case XFUNC_REQUIRE:
 		case XFUNC_REQUIRE_ONCE:
-			tmp_fname = xdebug_sprintf("%s::%s", tmp_name, ZSTR_VAL(fse->include_filename));
+			tmp_fname = xdebug_sprintf("%s::%s", tmp_name, fse->include_filename);
 			xdfree(tmp_name);
 			tmp_name = tmp_fname;
 			fse->profiler.lineno = 1;
@@ -352,9 +316,9 @@ void xdebug_profiler_add_function_details_user(function_stack_entry *fse, zend_o
 	}
 
 	if (op_array && op_array->filename) {
-		fse->profiler.filename = zend_string_copy(op_array->filename);
+		fse->profiler.filename = xdstrdup((char*) STR_NAME_VAL(op_array->filename));
 	} else {
-		fse->profiler.filename = zend_string_copy(fse->filename);
+		fse->profiler.filename = xdstrdup(fse->filename);
 	}
 	fse->profiler.funcname = xdstrdup(tmp_name);
 	xdfree(tmp_name);
@@ -384,7 +348,7 @@ void xdebug_profiler_add_function_details_internal(function_stack_entry *fse)
 		fse->profiler.lineno = 1;
 	}
 
-	fse->profiler.filename = zend_string_copy(fse->filename);
+	fse->profiler.filename = xdstrdup(fse->filename);
 	fse->profiler.funcname = xdstrdup(tmp_name);
 
 	xdfree(tmp_name);
@@ -392,79 +356,62 @@ void xdebug_profiler_add_function_details_internal(function_stack_entry *fse)
 
 void xdebug_profiler_function_begin(function_stack_entry *fse)
 {
-	fse->profile.nanotime = 0;
-	fse->profile.nanotime_mark = xdebug_get_nanotime();
+	fse->profile.time = 0;
+	fse->profile.mark = xdebug_get_utime();
 	fse->profile.memory = 0;
 	fse->profile.mem_mark = zend_memory_usage(0);
 }
 
-#define TMP_KEY_BUFFER_LEN 1024
-#define TMP_KEY_PREFIX     "php::"
-#define TMP_KEY_PREFIX_LEN (sizeof(TMP_KEY_PREFIX)-1)
-#define TMP_KEY_MAX_LEN    (TMP_KEY_BUFFER_LEN-TMP_KEY_PREFIX_LEN-1)
-
 void xdebug_profiler_function_end(function_stack_entry *fse)
 {
 	xdebug_llist_element *le;
-	xdebug_str file_buffer = XDEBUG_STR_INITIALIZER;
-	char tmp_key[TMP_KEY_BUFFER_LEN];
 
-	if (!XG_PROF(active)) {
-		return;
-	}
-
-	/* The temporary key always starts with 'php::' */
-	memcpy(tmp_key, TMP_KEY_PREFIX, TMP_KEY_PREFIX_LEN);
-
-	if (xdebug_vector_element_is_valid(XG_BASE(stack), fse - 1) && !(fse - 1)->profile.call_list) {
-		(fse - 1)->profile.call_list = xdebug_llist_alloc(xdebug_profile_call_entry_dtor);
+	if (fse->prev && !fse->prev->profile.call_list) {
+		fse->prev->profile.call_list = xdebug_llist_alloc(xdebug_profile_call_entry_dtor);
 	}
 	if (!fse->profile.call_list) {
 		fse->profile.call_list = xdebug_llist_alloc(xdebug_profile_call_entry_dtor);
 	}
 	xdebug_profiler_function_push(fse);
 
-	if (xdebug_vector_element_is_valid(XG_BASE(stack), fse - 1)) {
+	if (fse->prev) {
 		xdebug_call_entry *ce = xdmalloc(sizeof(xdebug_call_entry));
-
-		ce->filename = zend_string_copy(fse->profiler.filename);
+		ce->filename = xdstrdup(fse->profiler.filename);
 		ce->function = xdstrdup(fse->profiler.funcname);
-		ce->nanotime_taken = fse->profile.nanotime;
+		ce->time_taken = fse->profile.time;
 		ce->lineno = fse->lineno;
 		ce->user_defined = fse->user_defined;
 		ce->mem_used = fse->profile.memory;
 
-		xdebug_llist_insert_next((fse - 1)->profile.call_list, NULL, ce);
+		xdebug_llist_insert_next(fse->prev->profile.call_list, NULL, ce);
 	}
 
 	/* use previously created filename and funcname (or a reference to them) to show
 	 * time spend */
 	if (fse->user_defined == XDEBUG_BUILT_IN) {
-		size_t tmp_key_funcname_len = strlen(fse->profiler.funcname);
+		char *tmp_key = xdebug_sprintf("php::%s", fse->profiler.funcname);
+		char *fl_ref = NULL, *fn_ref = NULL;
 
-		memcpy(tmp_key + TMP_KEY_PREFIX_LEN,
-			fse->profiler.funcname,
-			tmp_key_funcname_len > TMP_KEY_MAX_LEN ? TMP_KEY_MAX_LEN : tmp_key_funcname_len + 1
-		);
-		tmp_key[TMP_KEY_BUFFER_LEN - 1] = '\0';
+		fl_ref = get_filename_ref((char*) "php:internal");
+		fn_ref = get_functionname_ref(tmp_key);
 
-		if (XG_PROF(php_internal_seen_before)) {
-			xdebug_str_add_literal(&file_buffer, "fl=(1)\n");
-		} else {
-			xdebug_str_add_literal(&file_buffer, "fl=(1) php:internal\n");
-			XG_PROF(php_internal_seen_before) = 1;
-		}
-		xdebug_str_add_literal(&file_buffer, "fn=");
-		add_functionname_ref(&file_buffer, tmp_key);
-		xdebug_str_addc(&file_buffer, '\n');
+		fprintf(XG_PROF(profile_file), "fl=%s\n", fl_ref);
+		fprintf(XG_PROF(profile_file), "fn=%s\n", fn_ref);
+
+		xdfree(fl_ref);
+		xdfree(fn_ref);
+		xdfree(tmp_key);
 	} else {
+		char *fl_ref = NULL, *fn_ref = NULL;
 
-		xdebug_str_add_literal(&file_buffer, "fl=");
-		add_filename_ref(&file_buffer, ZSTR_VAL(fse->profiler.filename));
+		fl_ref = get_filename_ref(fse->profiler.filename);
+		fn_ref = get_functionname_ref(fse->profiler.funcname);
 
-		xdebug_str_add_literal(&file_buffer, "\nfn=");
-		add_functionname_ref(&file_buffer, fse->profiler.funcname);
-		xdebug_str_addc(&file_buffer, '\n');
+		fprintf(XG_PROF(profile_file), "fl=%s\n", fl_ref);
+		fprintf(XG_PROF(profile_file), "fn=%s\n", fn_ref);
+
+		xdfree(fl_ref);
+		xdfree(fn_ref);
 	}
 
 
@@ -472,97 +419,68 @@ void xdebug_profiler_function_end(function_stack_entry *fse)
 	for (le = XDEBUG_LLIST_HEAD(fse->profile.call_list); le != NULL; le = XDEBUG_LLIST_NEXT(le))
 	{
 		xdebug_call_entry *call_entry = XDEBUG_LLIST_VALP(le);
-		fse->profile.nanotime -= call_entry->nanotime_taken;
+		fse->profile.time -= call_entry->time_taken;
 		fse->profile.memory -= call_entry->mem_used;
 	}
-
-	/* Adds %d %lu %lu, with lineno, time, and memory */
-	xdebug_str_add_uint64(&file_buffer, fse->profiler.lineno);
-	xdebug_str_addc(&file_buffer, ' ');
-	xdebug_str_add_uint64(&file_buffer, NANOTIME_SCALE_10NS(fse->profile.nanotime));
-	xdebug_str_addc(&file_buffer, ' ');
-	xdebug_str_add_uint64(&file_buffer, fse->profile.memory >= 0 ? fse->profile.memory : 0);
-	xdebug_str_addc(&file_buffer, '\n');
+	fprintf(XG_PROF(profile_file), "%d %lu %ld\n", fse->profiler.lineno, (unsigned long) (fse->profile.time * 1000000), (fse->profile.memory));
 
 	/* dump call list */
 	for (le = XDEBUG_LLIST_HEAD(fse->profile.call_list); le != NULL; le = XDEBUG_LLIST_NEXT(le))
 	{
+		char *fl_ref = NULL, *fn_ref = NULL;
 		xdebug_call_entry *call_entry = XDEBUG_LLIST_VALP(le);
 
 		if (call_entry->user_defined == XDEBUG_BUILT_IN) {
-			size_t tmp_key_funcname_len = strlen(call_entry->function);
+			char *tmp_key = xdebug_sprintf("php::%s", call_entry->function);
 
-			memcpy(tmp_key + TMP_KEY_PREFIX_LEN,
-				call_entry->function,
-				tmp_key_funcname_len > TMP_KEY_MAX_LEN ? TMP_KEY_MAX_LEN : tmp_key_funcname_len + 1
-			);
-			tmp_key[TMP_KEY_BUFFER_LEN - 1] = '\0';
+			fl_ref = get_filename_ref((char*) "php:internal");
+			fn_ref = get_functionname_ref(tmp_key);
 
-			if (XG_PROF(php_internal_seen_before)) {
-				xdebug_str_add_literal(&file_buffer, "cfl=(1)\n");
-			} else {
-				xdebug_str_add_literal(&file_buffer, "cfl=(1) php:internal\n");
-				XG_PROF(php_internal_seen_before) = 1;
-			}
-
-			xdebug_str_add_literal(&file_buffer, "cfn=");
-			add_functionname_ref(&file_buffer, tmp_key);
-			xdebug_str_addc(&file_buffer, '\n');
+			xdfree(tmp_key);
 		} else {
-			xdebug_str_add_literal(&file_buffer, "cfl=");
-			add_filename_ref(&file_buffer, ZSTR_VAL(call_entry->filename));
-
-			xdebug_str_add_literal(&file_buffer, "\ncfn=");
-			add_functionname_ref(&file_buffer, call_entry->function);
-			xdebug_str_addc(&file_buffer, '\n');
+			fl_ref = get_filename_ref(call_entry->filename);
+			fn_ref = get_functionname_ref(call_entry->function);
 		}
 
-		xdebug_str_add_literal(&file_buffer, "calls=1 0 0\n");
+		fprintf(XG_PROF(profile_file), "cfl=%s\n", fl_ref);
+		fprintf(XG_PROF(profile_file), "cfn=%s\n", fn_ref);
 
-		/* Adds %d %lu %lu, with lineno, time, and memory */
-		xdebug_str_add_uint64(&file_buffer, call_entry->lineno);
-		xdebug_str_addc(&file_buffer, ' ');
-		xdebug_str_add_uint64(&file_buffer, NANOTIME_SCALE_10NS(call_entry->nanotime_taken));
-		xdebug_str_addc(&file_buffer, ' ');
-		xdebug_str_add_uint64(&file_buffer, call_entry->mem_used >= 0 ? call_entry->mem_used : 0);
-		xdebug_str_addc(&file_buffer, '\n');
+		xdfree(fl_ref);
+		xdfree(fn_ref);
+
+		fprintf(XG_PROF(profile_file), "calls=1 0 0\n");
+		fprintf(XG_PROF(profile_file), "%d %lu %ld\n", call_entry->lineno, (unsigned long) (call_entry->time_taken * 1000000), (call_entry->mem_used));
 	}
-	xdebug_str_addc(&file_buffer, '\n');
-
-	fwrite(file_buffer.d, sizeof(char), file_buffer.l, XG_PROF(profile_file));
-	xdebug_str_dtor(file_buffer);
+	fprintf(XG_PROF(profile_file), "\n");
+	fflush(XG_PROF(profile_file));
 }
 
 void xdebug_profiler_free_function_details(function_stack_entry *fse)
 {
-	if (fse->profiler.funcname) {
-		xdfree(fse->profiler.funcname);
-		fse->profiler.funcname = NULL;
-	}
-	if (fse->profiler.filename) {
-		zend_string_release(fse->profiler.filename);
-		fse->profiler.filename = NULL;
-	}
+	xdfree(fse->profiler.funcname);
+	xdfree(fse->profiler.filename);
+	fse->profiler.funcname = NULL;
+	fse->profiler.filename = NULL;
 }
 
 /* Returns a *pointer* to the current profile filename, if active. NULL
  * otherwise. Calling function is responsible for duplicating immediately */
 char *xdebug_get_profiler_filename()
 {
-	if (!XG_PROF(active)) {
-		return NULL;
+	if (XG_PROF(profiler_enabled) && XG_PROF(profile_filename)) {
+		return XG_PROF(profile_filename);
 	}
 
-	return XG_PROF(profile_filename);
+	return NULL;
 }
 
 PHP_FUNCTION(xdebug_get_profiler_filename)
 {
 	char *filename = xdebug_get_profiler_filename();
 
-	if (!filename) {
+	if (filename) {
+		RETURN_STRING(filename);
+	} else {
 		RETURN_FALSE;
 	}
-
-	RETURN_STRING(filename);
 }
