@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002-2020 Derick Rethans                               |
+   | Copyright (c) 2002-2021 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 1.01 of the Xdebug license,   |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -12,8 +12,6 @@
    | to obtain it through the world-wide-web, please send a note to       |
    | derick@xdebug.org so we can mail you a copy immediately.             |
    +----------------------------------------------------------------------+
-   | Authors: Derick Rethans <derick@xdebug.org>                          |
-   +----------------------------------------------------------------------+
  */
 #include "php.h"
 #include "ext/standard/php_string.h"
@@ -30,11 +28,9 @@ extern ZEND_DECLARE_MODULE_GLOBALS(xdebug);
 void *xdebug_trace_computerized_init(char *fname, zend_string *script_filename, long options)
 {
 	xdebug_trace_computerized_context *tmp_computerized_context;
-	char *used_fname;
 
 	tmp_computerized_context = xdmalloc(sizeof(xdebug_trace_computerized_context));
-	tmp_computerized_context->trace_file = xdebug_trace_open_file(fname, script_filename, options, (char**) &used_fname);
-	tmp_computerized_context->trace_filename = used_fname;
+	tmp_computerized_context->trace_file = xdebug_trace_open_file(fname, script_filename, options);
 
 	return tmp_computerized_context->trace_file ? tmp_computerized_context : NULL;
 }
@@ -43,9 +39,9 @@ void xdebug_trace_computerized_deinit(void *ctxt)
 {
 	xdebug_trace_computerized_context *context = (xdebug_trace_computerized_context*) ctxt;
 
-	fclose(context->trace_file);
+	xdebug_file_close(context->trace_file);
+	xdebug_file_dtor(context->trace_file);
 	context->trace_file = NULL;
-	xdfree(context->trace_filename);
 
 	xdfree(context);
 }
@@ -55,14 +51,14 @@ void xdebug_trace_computerized_write_header(void *ctxt)
 	xdebug_trace_computerized_context *context = (xdebug_trace_computerized_context*) ctxt;
 	char *str_time;
 
-	fprintf(context->trace_file, "Version: %s\n", XDEBUG_VERSION);
-	fprintf(context->trace_file, "File format: 4\n");
+	xdebug_file_printf(context->trace_file, "Version: %s\n", XDEBUG_VERSION);
+	xdebug_file_printf(context->trace_file, "File format: 4\n");
 
 	str_time = xdebug_nanotime_to_chars(xdebug_get_nanotime(), 6);
-	fprintf(context->trace_file, "TRACE START [%s]\n", str_time);
+	xdebug_file_printf(context->trace_file, "TRACE START [%s]\n", str_time);
 	xdfree(str_time);
 
-	fflush(context->trace_file);
+	xdebug_file_flush(context->trace_file);
 }
 
 void xdebug_trace_computerized_write_footer(void *ctxt)
@@ -70,32 +66,25 @@ void xdebug_trace_computerized_write_footer(void *ctxt)
 	xdebug_trace_computerized_context *context = (xdebug_trace_computerized_context*) ctxt;
 	char    *str_time;
 	uint64_t nanotime;
-	char    *tmp;
 
 	nanotime = xdebug_get_nanotime();
 
-	tmp = xdebug_sprintf("\t\t\t%F\t", XDEBUG_SECONDS_SINCE_START(nanotime));
-	fprintf(context->trace_file, "%s", tmp);
-	xdfree(tmp);
-#if WIN32|WINNT
-	fprintf(context->trace_file, "%Iu", zend_memory_usage(0));
-#else
-	fprintf(context->trace_file, "%zu", zend_memory_usage(0));
-#endif
-	fprintf(context->trace_file, "\n");
+	xdebug_file_printf(context->trace_file, "\t\t\t%F\t", XDEBUG_SECONDS_SINCE_START(nanotime));
+	xdebug_file_printf(context->trace_file, "%zu", zend_memory_usage(0));
+	xdebug_file_printf(context->trace_file, "\n");
 
 	str_time = xdebug_nanotime_to_chars(xdebug_get_nanotime(), 6);
-	fprintf(context->trace_file, "TRACE END   [%s]\n\n", str_time);
+	xdebug_file_printf(context->trace_file, "TRACE END   [%s]\n\n", str_time);
 	xdfree(str_time);
 
-	fflush(context->trace_file);
+	xdebug_file_flush(context->trace_file);
 }
 
 char *xdebug_trace_computerized_get_filename(void *ctxt)
 {
 	xdebug_trace_computerized_context *context = (xdebug_trace_computerized_context*) ctxt;
 
-	return context->trace_filename;
+	return context->trace_file->name;
 }
 
 static void add_single_value(xdebug_str *str, zval *zv)
@@ -145,7 +134,7 @@ void xdebug_trace_computerized_function_entry(void *ctxt, function_stack_entry *
 	xdebug_str_add_fmt(&str, "%d\t", fse->level);
 	xdebug_str_add_fmt(&str, "%d\t", function_nr);
 
-	tmp_name = xdebug_show_fname(fse->function, 0, 0);
+	tmp_name = xdebug_show_fname(fse->function, XDEBUG_SHOW_FNAME_DEFAULT);
 
 	xdebug_str_add_literal(&str, "0\t");
 	xdebug_str_add_fmt(&str, "%F\t", XDEBUG_SECONDS_SINCE_START(fse->nanotime));
@@ -183,8 +172,8 @@ void xdebug_trace_computerized_function_entry(void *ctxt, function_stack_entry *
 	/* Trailing \n */
 	xdebug_str_addc(&str, '\n');
 
-	fprintf(context->trace_file, "%s", str.d);
-	fflush(context->trace_file);
+	xdebug_file_printf(context->trace_file, "%s", str.d);
+	xdebug_file_flush(context->trace_file);
 	xdfree(str.d);
 }
 
@@ -200,8 +189,8 @@ void xdebug_trace_computerized_function_exit(void *ctxt, function_stack_entry *f
 	xdebug_str_add_fmt(&str, "%F\t", XDEBUG_SECONDS_SINCE_START(xdebug_get_nanotime()));
 	xdebug_str_add_fmt(&str, "%lu\n", zend_memory_usage(0));
 
-	fprintf(context->trace_file, "%s", str.d);
-	fflush(context->trace_file);
+	xdebug_file_printf(context->trace_file, "%s", str.d);
+	xdebug_file_flush(context->trace_file);
 	xdfree(str.d);
 }
 
@@ -218,8 +207,50 @@ void xdebug_trace_computerized_function_return_value(void *ctxt, function_stack_
 
 	xdebug_str_add_literal(&str, "\n");
 
-	fprintf(context->trace_file, "%s", str.d);
-	fflush(context->trace_file);
+	xdebug_file_printf(context->trace_file, "%s", str.d);
+	xdebug_file_flush(context->trace_file);
+	xdfree(str.d);
+}
+
+void xdebug_trace_computerized_assignment(void *ctxt, function_stack_entry *fse, char *full_varname, zval *retval, char *right_full_varname, const char *op, char *filename, int lineno)
+{
+	xdebug_trace_computerized_context *context = (xdebug_trace_computerized_context*) ctxt;
+	xdebug_str                         str = XDEBUG_STR_INITIALIZER;
+	xdebug_str                        *tmp_value;
+
+	xdebug_str_add_fmt(&str, "%d\t", fse->level);
+	/* no function_nr */
+	xdebug_str_add_literal(&str, "\t");
+
+	xdebug_str_add_literal(&str, "A\t");
+	/* skip time index, memory usage, function name, user defined */
+	xdebug_str_add_literal(&str, "\t\t\t\t");
+
+	/* Filename and Lineno (9, 10) */
+	xdebug_str_add_fmt(&str, "\t%s\t%d", filename, lineno);
+	xdebug_str_add_fmt(&str, "\t%s", full_varname);
+
+	if (op[0] != '\0' ) { /* pre/post inc/dec ops are special */
+		xdebug_str_addc(&str, ' ');
+		xdebug_str_add(&str, op, 0);
+		xdebug_str_addc(&str, ' ');
+
+		tmp_value = xdebug_get_zval_value_line(retval, 0, NULL);
+
+		if (tmp_value) {
+			xdebug_str_add_str(&str, tmp_value);
+			xdebug_str_free(tmp_value);
+		} else {
+			xdebug_str_add_literal(&str, "NULL");
+		}
+	}
+
+	/* Trailing \n */
+	xdebug_str_add_literal(&str, "\n");
+
+	xdebug_file_printf(context->trace_file, "%s", str.d);
+	xdebug_file_flush(context->trace_file);
+
 	xdfree(str.d);
 }
 
@@ -234,5 +265,5 @@ xdebug_trace_handler_t xdebug_trace_handler_computerized =
 	xdebug_trace_computerized_function_exit,
 	xdebug_trace_computerized_function_return_value,
 	NULL /* xdebug_trace_computerized_generator_return_value */,
-	NULL /* xdebug_trace_computerized_assignment */
+	xdebug_trace_computerized_assignment
 };
