@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002-2020 Derick Rethans                               |
+   | Copyright (c) 2002-2021 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 1.01 of the Xdebug license,   |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -12,8 +12,6 @@
    | to obtain it through the world-wide-web, please send a note to       |
    | derick@xdebug.org so we can mail you a copy immediately.             |
    +----------------------------------------------------------------------+
-   | Authors: Derick Rethans <derick@xdebug.org>                          |
-   +----------------------------------------------------------------------+
  */
 #include "php.h"
 #include "ext/standard/php_string.h"
@@ -54,9 +52,9 @@ static xdebug_trace_handler_t *xdebug_select_trace_handler(int options)
 	return tmp;
 }
 
-FILE *xdebug_trace_open_file(char *requested_filename, zend_string *script_filename, long options, char **used_fname)
+xdebug_file *xdebug_trace_open_file(char *requested_filename, zend_string *script_filename, long options)
 {
-	FILE *file;
+	xdebug_file *file = xdebug_file_ctor();
 	char *filename_to_use;
 	char *generated_filename = NULL;
 	char *output_dir = xdebug_lib_get_output_dir(); /* not duplicated */
@@ -81,13 +79,13 @@ FILE *xdebug_trace_open_file(char *requested_filename, zend_string *script_filen
 			filename_to_use = xdebug_sprintf("%s%c%s", output_dir, DEFAULT_SLASH, generated_filename);
 		}
 	}
-	if (options & XDEBUG_TRACE_OPTION_APPEND) {
-		file = xdebug_fopen(filename_to_use, "a", (options & XDEBUG_TRACE_OPTION_NAKED_FILENAME) ? NULL : "xt", used_fname);
-	} else {
-		file = xdebug_fopen(filename_to_use, "w", (options & XDEBUG_TRACE_OPTION_NAKED_FILENAME) ? NULL : "xt", used_fname);
-	}
 
-	if (!file) {
+	if (!xdebug_file_open(
+		file,
+		filename_to_use,
+		(options & XDEBUG_TRACE_OPTION_NAKED_FILENAME) ? NULL : "xt",
+		(options & XDEBUG_TRACE_OPTION_APPEND) ? "a" : "w"
+	)) {
 		xdebug_log_diagnose_permissions(XLOG_CHAN_TRACE, output_dir, generated_filename);
 	}
 
@@ -217,7 +215,7 @@ static const char *get_assign_operation(uint32_t extended_value)
 }
 #endif
 
-static int xdebug_is_static_call(const zend_op *cur_opcode, const zend_op *prev_opcode, const zend_op **found_opcode)
+static int xdebug_is_static_call(const zend_op *first_opcode, const zend_op *cur_opcode, const zend_op *prev_opcode, const zend_op **found_opcode)
 {
 	const zend_op *opcode_ptr;
 
@@ -235,6 +233,9 @@ static int xdebug_is_static_call(const zend_op *cur_opcode, const zend_op *prev_
 # endif
 	while (!(opcode_ptr->opcode == ZEND_EXT_STMT) && !((opcode_ptr->opcode == ZEND_FETCH_STATIC_PROP_W) || (opcode_ptr->opcode == ZEND_FETCH_STATIC_PROP_RW))) {
 		opcode_ptr = opcode_ptr - 1;
+		if (opcode_ptr < first_opcode) {
+			return 0;
+		}
 	}
 	if ((opcode_ptr->opcode == ZEND_FETCH_STATIC_PROP_W) || (opcode_ptr->opcode == ZEND_FETCH_STATIC_PROP_RW)) {
 		*found_opcode = opcode_ptr;
@@ -296,9 +297,11 @@ static char *xdebug_find_var_name(zend_execute_data *execute_data, const zend_op
 	if (cur_opcode->opcode == ZEND_QM_ASSIGN) {
 		xdebug_str_addc(&name, '$');
 		xdebug_str_add(&name, zend_get_compiled_variable_name(op_array, cur_opcode->result.var)->val, 0);
+
+		return name.d;
 	}
 
-	is_static = xdebug_is_static_call(cur_opcode, prev_opcode, &static_opcode_ptr);
+	is_static = xdebug_is_static_call(op_array->opcodes, cur_opcode, prev_opcode, &static_opcode_ptr);
 	options = xdebug_var_export_options_from_ini();
 	options->no_decoration = 1;
 
@@ -581,15 +584,15 @@ static int xdebug_common_assign_dim_handler(const char *op, XDEBUG_OPCODE_HANDLE
 			}
 #if PHP_VERSION_ID >= 70400
 		} else if (cur_opcode->opcode == ZEND_ASSIGN_OBJ_REF) {
-			if (cur_opcode->op2_type == IS_CV) {
-				right_full_varname = xdebug_sprintf("$%s", zend_get_compiled_variable_name(op_array, cur_opcode->op2.var)->val);
+			if (next_opcode->op1_type == IS_CV) {
+				right_full_varname = xdebug_sprintf("$%s", zend_get_compiled_variable_name(op_array, next_opcode->op1.var)->val);
 			} else {
 				const zend_op *referenced_opline = xdebug_find_referenced_opline(execute_data, next_opcode, 1);
 				right_full_varname = xdebug_find_var_name(execute_data, referenced_opline, NULL);
 			}
 		} else if (cur_opcode->opcode == ZEND_ASSIGN_STATIC_PROP_REF) {
-			if (cur_opcode->op2_type == IS_CV) {
-				right_full_varname = xdebug_sprintf("$%s", zend_get_compiled_variable_name(op_array, cur_opcode->op2.var)->val);
+			if (next_opcode->op1_type == IS_CV) {
+				right_full_varname = xdebug_sprintf("$%s", zend_get_compiled_variable_name(op_array, next_opcode->op1.var)->val);
 			} else {
 				const zend_op *referenced_opline = xdebug_find_referenced_opline(execute_data, next_opcode, 1);
 				right_full_varname = xdebug_find_var_name(execute_data, referenced_opline, NULL);
@@ -730,7 +733,7 @@ void xdebug_tracing_post_deactivate(void)
 
 void xdebug_tracing_init_if_requested(zend_op_array *op_array)
 {
-	if (xdebug_lib_start_with_request() || xdebug_lib_start_with_trigger(XDEBUG_MODE_TRACING, NULL)) {
+	if (xdebug_lib_start_with_request(XDEBUG_MODE_TRACING) || xdebug_lib_start_with_trigger(XDEBUG_MODE_TRACING, NULL)) {
 		/* In case we do an auto-trace we are not interested in the return
 		 * value, but we still have to free it. */
 		xdfree(xdebug_start_trace(NULL, op_array->filename, XINI_TRACE(trace_options)));
