@@ -19,15 +19,11 @@
 #define XDEBUG_TRACING_H
 
 #include "php.h"
-
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wdeclaration-after-statement"
 #include "zend_generators.h"
-#pragma GCC diagnostic pop
 
 typedef struct
 {
-	void *(*init)(char *fname, zend_string *script_filename, long options);
+	void *(*init)(char *fname, char *script_filename, long options);
 	void (*deinit)(void *ctxt);
 	void (*write_header)(void *ctxt);
 	void (*write_footer)(void *ctxt);
@@ -45,19 +41,19 @@ typedef struct _xdebug_tracing_globals_t {
 } xdebug_tracing_globals_t;
 
 typedef struct _xdebug_tracing_settings_t {
+	zend_bool     auto_trace;
+	zend_bool     trace_enable_trigger;
+	char         *trace_enable_trigger_value;
+	char         *trace_output_dir;
 	char         *trace_output_name;
 	zend_long     trace_options;
 	zend_long     trace_format;
-
-	zend_bool     collect_assignments;
-	zend_bool     collect_return;
 } xdebug_tracing_settings_t;
 
 void xdebug_init_tracing_globals(xdebug_tracing_globals_t *xg);
 void xdebug_tracing_minit(INIT_FUNC_ARGS);
 void xdebug_tracing_rinit(void);
 void xdebug_tracing_post_deactivate(void);
-void xdebug_tracing_register_constants(INIT_FUNC_ARGS);
 
 void xdebug_tracing_init_if_requested(zend_op_array *op_array);
 void xdebug_tracing_execute_ex(int function_nr, function_stack_entry *fse);
@@ -74,6 +70,4 @@ char* xdebug_return_trace_assignment(function_stack_entry *i, char *varname, zva
 
 void xdebug_trace_function_begin(function_stack_entry *fse, int function_nr);
 void xdebug_trace_function_end(function_stack_entry *fse, int function_nr);
-
-char *xdebug_get_trace_filename(void);
 #endif
