@@ -2,26 +2,24 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002-2018 Derick Rethans                               |
+   | Copyright (c) 2002-2016 Derick Rethans                               |
    +----------------------------------------------------------------------+
-   | This source file is subject to version 1.01 of the Xdebug license,   |
+   | This source file is subject to version 1.0 of the Xdebug license,    |
    | that is bundled with this package in the file LICENSE, and is        |
    | available at through the world-wide-web at                           |
-   | https://xdebug.org/license.php                                       |
+   | http://xdebug.derickrethans.nl/license.php                           |
    | If you did not receive a copy of the Xdebug license and are unable   |
    | to obtain it through the world-wide-web, please send a note to       |
-   | derick@xdebug.org so we can mail you a copy immediately.             |
+   | xdebug@derickrethans.nl so we can mail you a copy immediately.       |
    +----------------------------------------------------------------------+
-   | Authors: Derick Rethans <derick@xdebug.org>                          |
-   |          Thomas Vanhaniemi <thomas.vanhaniemi@arcada.fi>             |
+   | Authors:  Derick Rethans <derick@xdebug.org>                         |
+   |           Thomas Vanhaniemi <thomas.vanhaniemi@arcada.fi>            |
    +----------------------------------------------------------------------+
  */
 
 #ifndef __HAVE_XDEBUG_COM_H__
 #define __HAVE_XDEBUG_COM_H__
 
-#include "xdebug_handlers.h"
-
 #if WIN32|WINNT
 # define SOCK_ERR INVALID_SOCKET
 # define SOCK_CONN_ERR SOCKET_ERROR
@@ -55,20 +53,9 @@
 	xdfree(message_buffer);     \
 }
 
-int xdebug_create_socket(const char *hostname, int dport, int timeout TSRMLS_DC);
+#define XDEBUG_LOG_PRINT(fs, string, ...) if (fs) { fprintf(fs, string, ## __VA_ARGS__); }
+
+int xdebug_create_socket(const char *hostname, int dport TSRMLS_DC);
 void xdebug_close_socket(int socket);
 
-/* Remote debugging helper functions */
-int xdebug_handle_hit_value(xdebug_brk_info *brk_info);
-
-/* Remote connection activation and house keeping */
-int xdebug_is_debug_connection_active(void);
-int xdebug_is_debug_connection_active_for_current_pid(void);
-void xdebug_stop_debugger(void);
-void xdebug_mark_debug_connection_active(void);
-void xdebug_mark_debug_connection_not_active(void);
-void xdebug_mark_debug_connection_pending(void);
-void xdebug_do_jit(TSRMLS_D);
-void xdebug_do_req();
-
 #endif
